# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Repositories.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_ModelF_ice
import omero_ServicesF_ice
import omero_System_ice
import omero_Collections_ice
import omero_ServerErrors_ice
import omero_cmd_API_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')

# Included module omero.cmd
_M_omero.cmd = Ice.openModule('omero.cmd')

# Start of module omero
__name__ = 'omero'

# Start of module omero.grid
__name__ = 'omero.grid'

if 'RepositoryException' not in _M_omero.grid.__dict__:
    _M_omero.grid.RepositoryException = Ice.createTempClass()
    class RepositoryException(_M_omero.ServerError):
        """
        Base repository exception.
        """
        def __init__(self, serverStackTrace='', serverExceptionClass='', message=''):
            _M_omero.ServerError.__init__(self, serverStackTrace, serverExceptionClass, message)

        def __str__(self):
            return IcePy.stringifyException(self)

        __repr__ = __str__

        _ice_name = 'omero::grid::RepositoryException'

    _M_omero.grid._t_RepositoryException = IcePy.defineException('::omero::grid::RepositoryException', RepositoryException, (), False, _M_omero._t_ServerError, ())
    RepositoryException._ice_type = _M_omero.grid._t_RepositoryException

    _M_omero.grid.RepositoryException = RepositoryException
    del RepositoryException

if 'FileDeleteException' not in _M_omero.grid.__dict__:
    _M_omero.grid.FileDeleteException = Ice.createTempClass()
    class FileDeleteException(_M_omero.grid.RepositoryException):
        """
        Specifies that a file with the given path has failed to
        be deleted from the file system.
        """
        def __init__(self, serverStackTrace='', serverExceptionClass='', message=''):
            _M_omero.grid.RepositoryException.__init__(self, serverStackTrace, serverExceptionClass, message)

        def __str__(self):
            return IcePy.stringifyException(self)

        __repr__ = __str__

        _ice_name = 'omero::grid::FileDeleteException'

    _M_omero.grid._t_FileDeleteException = IcePy.defineException('::omero::grid::FileDeleteException', FileDeleteException, (), False, _M_omero.grid._t_RepositoryException, ())
    FileDeleteException._ice_type = _M_omero.grid._t_FileDeleteException

    _M_omero.grid.FileDeleteException = FileDeleteException
    del FileDeleteException

if 'UnregisteredFileException' not in _M_omero.grid.__dict__:
    _M_omero.grid.UnregisteredFileException = Ice.createTempClass()
    class UnregisteredFileException(_M_omero.grid.RepositoryException):
        """
        Specifies that a file is located at the given location
        that is not otherwise known by the repository. A
        subsequent call to {@code Repository.register} will create
        the given file. The mimetype field of the file may or
        may not be set. If it is set, clients are suggested to
        either omit the mimetype argument to the register method
        or to pass the same value.
        """
        def __init__(self, serverStackTrace='', serverExceptionClass='', message='', file=None):
            _M_omero.grid.RepositoryException.__init__(self, serverStackTrace, serverExceptionClass, message)
            self.file = file

        def __str__(self):
            return IcePy.stringifyException(self)

        __repr__ = __str__

        _ice_name = 'omero::grid::UnregisteredFileException'

    _M_omero.grid._t_UnregisteredFileException = IcePy.defineException('::omero::grid::UnregisteredFileException', UnregisteredFileException, (), False, _M_omero.grid._t_RepositoryException, (('file', (), _M_omero.model._t_OriginalFile, False, 0),))
    UnregisteredFileException._ice_type = _M_omero.grid._t_UnregisteredFileException

    _M_omero.grid.UnregisteredFileException = UnregisteredFileException
    del UnregisteredFileException

if 'Repository' not in _M_omero.grid.__dict__:
    _M_omero.grid.Repository = Ice.createTempClass()
    class Repository(Ice.Object):
        """
        Client-accessible interface representing a single mount point on the server-side.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.grid.Repository:
                raise RuntimeError('omero.grid.Repository is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::grid::Repository')

        def ice_id(self, current=None):
            return '::omero::grid::Repository'

        def ice_staticId():
            return '::omero::grid::Repository'
        ice_staticId = staticmethod(ice_staticId)

        def root(self, current=None):
            """
            Return the OriginalFile descriptor for this Repository. It will have
            the path of the repository's root on the underlying filesystem.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def mimetype(self, path, current=None):
            """
            Returns the best-guess mimetype for the given path.
            Arguments:
            path -- 
            current -- The Current object for the invocation.
            """
            pass

        def list(self, path, current=None):
            """
            Returns a set of strings naming the files and directories in
            the directory denoted by an abstract pathname.
            Arguments:
            path -- 
            current -- The Current object for the invocation.
            """
            pass

        def listFiles(self, path, current=None):
            """
            Returns an array of abstract pathname objects denoting the
            files in the directory denoted by an abstract pathname.  It
            is expected that at a minimum the ""name"", ""path"", ""size"" and
            ""mtime"" attributes will be present for each
            omero.model.OriginalFile instance.
            Arguments:
            path -- 
            current -- The Current object for the invocation.
            """
            pass

        def register(self, path, mimetype, current=None):
            """
            Create an OriginalFile in the database for the given path.
            Arguments:
            path -- 
            mimetype -- 
            current -- The Current object for the invocation.
            """
            pass

        def file(self, path, mode, current=None):
            """
            Returns a special RawFileStore which permits only the operations
            set out in the options string ""wb"", ""a+"", etc.
            FIXME: Initially only ""r"" and ""rw"" are supported as these are
            handled directly by RandomAccessFile and so don't break the current
            implementation.
            Any call to that tries to break the options will throw an
            ApiUsageException. If a file exists at the given path, a
            ValidationException will be thrown.
            Arguments:
            path -- 
            mode -- 
            current -- The Current object for the invocation.
            """
            pass

        def fileById(self, id, current=None):
            pass

        def fileExists(self, path, current=None):
            """
            Returns true if the file or path exists within the repository.
            In other words, if a call on `dirname path` to
            {@code listFiles} would return an object for this path.
            Arguments:
            path -- 
            current -- The Current object for the invocation.
            """
            pass

        def makeDir(self, path, parents, current=None):
            """
            Create a directory at the given path. If parents is true,
            then all preceding paths will be generated and no exception
            will be thrown if the directory already exists. Otherwise,
            all parent directories must exist in both the DB and on the
            filesystem and be readable.
            Arguments:
            path -- 
            parents -- 
            current -- The Current object for the invocation.
            """
            pass

        def treeList(self, path, current=None):
            """
            Similar to {@code list} but recursive and returns only
            primitive values for the file at each location. Guaranteed for
            each path is only the values id and mimetype.
            After a call to unwrap, the returned omero.RMap for a
            call to treeList("/user_1/dir0") might look something like:
            {@code
            {
            "/user_1/dir0/file1.txt" :
            {
            "id":10,
            "mimetype":
            "binary",
            "size": 10000L
            },
            "/user_1/dir0/dir1" :
            {
            "id": 100,
            "mimetype": "Directory",
            "size": 0L,
            "files":
            {
            "/user_1/dir0/dir1/file1indir.txt" :
            {
            "id": 1,
            "mimetype": "png",
            "size": 500
            }
            }
            }
            }
            }
            Arguments:
            path -- 
            current -- The Current object for the invocation.
            """
            pass

        def deletePaths(self, paths, recursively, force, current=None):
            """
            Delete several individual paths. Internally, this converts
            all of the paths into a single omero.cmd.Delete2 command
            and submits it.
            If a ""recursively"" is true, then directories will be searched
            and all of their contained files will be placed before them in
            the delete order. When the directory is removed from the database,
            it will removed from the filesystem if and only if it is empty.
            If ""recursively"" is false, then the delete will produce an error
            according to the ""force"" flag.
            If ""force"" is false, this method attempts the delete of all given
            paths in a single transaction, and any failure will cause the
            entire transaction to fail.
            If ""force"" is true, however, then all the other deletes will succeed.
            which could possibly leave dangling files within no longer extant
            directories.
            Arguments:
            paths -- 
            recursively -- 
            force -- 
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid._t_Repository)

        __repr__ = __str__

    _M_omero.grid.RepositoryPrx = Ice.createTempClass()
    class RepositoryPrx(Ice.ObjectPrx):

        """
        Return the OriginalFile descriptor for this Repository. It will have
        the path of the repository's root on the underlying filesystem.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def root(self, _ctx=None):
            return _M_omero.grid.Repository._op_root.invoke(self, ((), _ctx))

        """
        Return the OriginalFile descriptor for this Repository. It will have
        the path of the repository's root on the underlying filesystem.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_root(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.Repository._op_root.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Return the OriginalFile descriptor for this Repository. It will have
        the path of the repository's root on the underlying filesystem.
        Arguments:
        """
        def end_root(self, _r):
            return _M_omero.grid.Repository._op_root.end(self, _r)

        """
        Returns the best-guess mimetype for the given path.
        Arguments:
        path -- 
        _ctx -- The request context for the invocation.
        """
        def mimetype(self, path, _ctx=None):
            return _M_omero.grid.Repository._op_mimetype.invoke(self, ((path, ), _ctx))

        """
        Returns the best-guess mimetype for the given path.
        Arguments:
        path -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_mimetype(self, path, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.Repository._op_mimetype.begin(self, ((path, ), _response, _ex, _sent, _ctx))

        """
        Returns the best-guess mimetype for the given path.
        Arguments:
        path -- 
        """
        def end_mimetype(self, _r):
            return _M_omero.grid.Repository._op_mimetype.end(self, _r)

        """
        Returns a set of strings naming the files and directories in
        the directory denoted by an abstract pathname.
        Arguments:
        path -- 
        _ctx -- The request context for the invocation.
        """
        def list(self, path, _ctx=None):
            return _M_omero.grid.Repository._op_list.invoke(self, ((path, ), _ctx))

        """
        Returns a set of strings naming the files and directories in
        the directory denoted by an abstract pathname.
        Arguments:
        path -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_list(self, path, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.Repository._op_list.begin(self, ((path, ), _response, _ex, _sent, _ctx))

        """
        Returns a set of strings naming the files and directories in
        the directory denoted by an abstract pathname.
        Arguments:
        path -- 
        """
        def end_list(self, _r):
            return _M_omero.grid.Repository._op_list.end(self, _r)

        """
        Returns an array of abstract pathname objects denoting the
        files in the directory denoted by an abstract pathname.  It
        is expected that at a minimum the ""name"", ""path"", ""size"" and
        ""mtime"" attributes will be present for each
        omero.model.OriginalFile instance.
        Arguments:
        path -- 
        _ctx -- The request context for the invocation.
        """
        def listFiles(self, path, _ctx=None):
            return _M_omero.grid.Repository._op_listFiles.invoke(self, ((path, ), _ctx))

        """
        Returns an array of abstract pathname objects denoting the
        files in the directory denoted by an abstract pathname.  It
        is expected that at a minimum the ""name"", ""path"", ""size"" and
        ""mtime"" attributes will be present for each
        omero.model.OriginalFile instance.
        Arguments:
        path -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_listFiles(self, path, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.Repository._op_listFiles.begin(self, ((path, ), _response, _ex, _sent, _ctx))

        """
        Returns an array of abstract pathname objects denoting the
        files in the directory denoted by an abstract pathname.  It
        is expected that at a minimum the ""name"", ""path"", ""size"" and
        ""mtime"" attributes will be present for each
        omero.model.OriginalFile instance.
        Arguments:
        path -- 
        """
        def end_listFiles(self, _r):
            return _M_omero.grid.Repository._op_listFiles.end(self, _r)

        """
        Create an OriginalFile in the database for the given path.
        Arguments:
        path -- 
        mimetype -- 
        _ctx -- The request context for the invocation.
        """
        def register(self, path, mimetype, _ctx=None):
            return _M_omero.grid.Repository._op_register.invoke(self, ((path, mimetype), _ctx))

        """
        Create an OriginalFile in the database for the given path.
        Arguments:
        path -- 
        mimetype -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_register(self, path, mimetype, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.Repository._op_register.begin(self, ((path, mimetype), _response, _ex, _sent, _ctx))

        """
        Create an OriginalFile in the database for the given path.
        Arguments:
        path -- 
        mimetype -- 
        """
        def end_register(self, _r):
            return _M_omero.grid.Repository._op_register.end(self, _r)

        """
        Returns a special RawFileStore which permits only the operations
        set out in the options string ""wb"", ""a+"", etc.
        FIXME: Initially only ""r"" and ""rw"" are supported as these are
        handled directly by RandomAccessFile and so don't break the current
        implementation.
        Any call to that tries to break the options will throw an
        ApiUsageException. If a file exists at the given path, a
        ValidationException will be thrown.
        Arguments:
        path -- 
        mode -- 
        _ctx -- The request context for the invocation.
        """
        def file(self, path, mode, _ctx=None):
            return _M_omero.grid.Repository._op_file.invoke(self, ((path, mode), _ctx))

        """
        Returns a special RawFileStore which permits only the operations
        set out in the options string ""wb"", ""a+"", etc.
        FIXME: Initially only ""r"" and ""rw"" are supported as these are
        handled directly by RandomAccessFile and so don't break the current
        implementation.
        Any call to that tries to break the options will throw an
        ApiUsageException. If a file exists at the given path, a
        ValidationException will be thrown.
        Arguments:
        path -- 
        mode -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_file(self, path, mode, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.Repository._op_file.begin(self, ((path, mode), _response, _ex, _sent, _ctx))

        """
        Returns a special RawFileStore which permits only the operations
        set out in the options string ""wb"", ""a+"", etc.
        FIXME: Initially only ""r"" and ""rw"" are supported as these are
        handled directly by RandomAccessFile and so don't break the current
        implementation.
        Any call to that tries to break the options will throw an
        ApiUsageException. If a file exists at the given path, a
        ValidationException will be thrown.
        Arguments:
        path -- 
        mode -- 
        """
        def end_file(self, _r):
            return _M_omero.grid.Repository._op_file.end(self, _r)

        def fileById(self, id, _ctx=None):
            return _M_omero.grid.Repository._op_fileById.invoke(self, ((id, ), _ctx))

        def begin_fileById(self, id, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.Repository._op_fileById.begin(self, ((id, ), _response, _ex, _sent, _ctx))

        def end_fileById(self, _r):
            return _M_omero.grid.Repository._op_fileById.end(self, _r)

        """
        Returns true if the file or path exists within the repository.
        In other words, if a call on `dirname path` to
        {@code listFiles} would return an object for this path.
        Arguments:
        path -- 
        _ctx -- The request context for the invocation.
        """
        def fileExists(self, path, _ctx=None):
            return _M_omero.grid.Repository._op_fileExists.invoke(self, ((path, ), _ctx))

        """
        Returns true if the file or path exists within the repository.
        In other words, if a call on `dirname path` to
        {@code listFiles} would return an object for this path.
        Arguments:
        path -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_fileExists(self, path, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.Repository._op_fileExists.begin(self, ((path, ), _response, _ex, _sent, _ctx))

        """
        Returns true if the file or path exists within the repository.
        In other words, if a call on `dirname path` to
        {@code listFiles} would return an object for this path.
        Arguments:
        path -- 
        """
        def end_fileExists(self, _r):
            return _M_omero.grid.Repository._op_fileExists.end(self, _r)

        """
        Create a directory at the given path. If parents is true,
        then all preceding paths will be generated and no exception
        will be thrown if the directory already exists. Otherwise,
        all parent directories must exist in both the DB and on the
        filesystem and be readable.
        Arguments:
        path -- 
        parents -- 
        _ctx -- The request context for the invocation.
        """
        def makeDir(self, path, parents, _ctx=None):
            return _M_omero.grid.Repository._op_makeDir.invoke(self, ((path, parents), _ctx))

        """
        Create a directory at the given path. If parents is true,
        then all preceding paths will be generated and no exception
        will be thrown if the directory already exists. Otherwise,
        all parent directories must exist in both the DB and on the
        filesystem and be readable.
        Arguments:
        path -- 
        parents -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_makeDir(self, path, parents, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.Repository._op_makeDir.begin(self, ((path, parents), _response, _ex, _sent, _ctx))

        """
        Create a directory at the given path. If parents is true,
        then all preceding paths will be generated and no exception
        will be thrown if the directory already exists. Otherwise,
        all parent directories must exist in both the DB and on the
        filesystem and be readable.
        Arguments:
        path -- 
        parents -- 
        """
        def end_makeDir(self, _r):
            return _M_omero.grid.Repository._op_makeDir.end(self, _r)

        """
        Similar to {@code list} but recursive and returns only
        primitive values for the file at each location. Guaranteed for
        each path is only the values id and mimetype.
        After a call to unwrap, the returned omero.RMap for a
        call to treeList("/user_1/dir0") might look something like:
        {@code
        {
        "/user_1/dir0/file1.txt" :
        {
        "id":10,
        "mimetype":
        "binary",
        "size": 10000L
        },
        "/user_1/dir0/dir1" :
        {
        "id": 100,
        "mimetype": "Directory",
        "size": 0L,
        "files":
        {
        "/user_1/dir0/dir1/file1indir.txt" :
        {
        "id": 1,
        "mimetype": "png",
        "size": 500
        }
        }
        }
        }
        }
        Arguments:
        path -- 
        _ctx -- The request context for the invocation.
        """
        def treeList(self, path, _ctx=None):
            return _M_omero.grid.Repository._op_treeList.invoke(self, ((path, ), _ctx))

        """
        Similar to {@code list} but recursive and returns only
        primitive values for the file at each location. Guaranteed for
        each path is only the values id and mimetype.
        After a call to unwrap, the returned omero.RMap for a
        call to treeList("/user_1/dir0") might look something like:
        {@code
        {
        "/user_1/dir0/file1.txt" :
        {
        "id":10,
        "mimetype":
        "binary",
        "size": 10000L
        },
        "/user_1/dir0/dir1" :
        {
        "id": 100,
        "mimetype": "Directory",
        "size": 0L,
        "files":
        {
        "/user_1/dir0/dir1/file1indir.txt" :
        {
        "id": 1,
        "mimetype": "png",
        "size": 500
        }
        }
        }
        }
        }
        Arguments:
        path -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_treeList(self, path, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.Repository._op_treeList.begin(self, ((path, ), _response, _ex, _sent, _ctx))

        """
        Similar to {@code list} but recursive and returns only
        primitive values for the file at each location. Guaranteed for
        each path is only the values id and mimetype.
        After a call to unwrap, the returned omero.RMap for a
        call to treeList("/user_1/dir0") might look something like:
        {@code
        {
        "/user_1/dir0/file1.txt" :
        {
        "id":10,
        "mimetype":
        "binary",
        "size": 10000L
        },
        "/user_1/dir0/dir1" :
        {
        "id": 100,
        "mimetype": "Directory",
        "size": 0L,
        "files":
        {
        "/user_1/dir0/dir1/file1indir.txt" :
        {
        "id": 1,
        "mimetype": "png",
        "size": 500
        }
        }
        }
        }
        }
        Arguments:
        path -- 
        """
        def end_treeList(self, _r):
            return _M_omero.grid.Repository._op_treeList.end(self, _r)

        """
        Delete several individual paths. Internally, this converts
        all of the paths into a single omero.cmd.Delete2 command
        and submits it.
        If a ""recursively"" is true, then directories will be searched
        and all of their contained files will be placed before them in
        the delete order. When the directory is removed from the database,
        it will removed from the filesystem if and only if it is empty.
        If ""recursively"" is false, then the delete will produce an error
        according to the ""force"" flag.
        If ""force"" is false, this method attempts the delete of all given
        paths in a single transaction, and any failure will cause the
        entire transaction to fail.
        If ""force"" is true, however, then all the other deletes will succeed.
        which could possibly leave dangling files within no longer extant
        directories.
        Arguments:
        paths -- 
        recursively -- 
        force -- 
        _ctx -- The request context for the invocation.
        """
        def deletePaths(self, paths, recursively, force, _ctx=None):
            return _M_omero.grid.Repository._op_deletePaths.invoke(self, ((paths, recursively, force), _ctx))

        """
        Delete several individual paths. Internally, this converts
        all of the paths into a single omero.cmd.Delete2 command
        and submits it.
        If a ""recursively"" is true, then directories will be searched
        and all of their contained files will be placed before them in
        the delete order. When the directory is removed from the database,
        it will removed from the filesystem if and only if it is empty.
        If ""recursively"" is false, then the delete will produce an error
        according to the ""force"" flag.
        If ""force"" is false, this method attempts the delete of all given
        paths in a single transaction, and any failure will cause the
        entire transaction to fail.
        If ""force"" is true, however, then all the other deletes will succeed.
        which could possibly leave dangling files within no longer extant
        directories.
        Arguments:
        paths -- 
        recursively -- 
        force -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_deletePaths(self, paths, recursively, force, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.Repository._op_deletePaths.begin(self, ((paths, recursively, force), _response, _ex, _sent, _ctx))

        """
        Delete several individual paths. Internally, this converts
        all of the paths into a single omero.cmd.Delete2 command
        and submits it.
        If a ""recursively"" is true, then directories will be searched
        and all of their contained files will be placed before them in
        the delete order. When the directory is removed from the database,
        it will removed from the filesystem if and only if it is empty.
        If ""recursively"" is false, then the delete will produce an error
        according to the ""force"" flag.
        If ""force"" is false, this method attempts the delete of all given
        paths in a single transaction, and any failure will cause the
        entire transaction to fail.
        If ""force"" is true, however, then all the other deletes will succeed.
        which could possibly leave dangling files within no longer extant
        directories.
        Arguments:
        paths -- 
        recursively -- 
        force -- 
        """
        def end_deletePaths(self, _r):
            return _M_omero.grid.Repository._op_deletePaths.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.RepositoryPrx.ice_checkedCast(proxy, '::omero::grid::Repository', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.RepositoryPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::Repository'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid._t_RepositoryPrx = IcePy.defineProxy('::omero::grid::Repository', RepositoryPrx)

    _M_omero.grid._t_Repository = IcePy.defineClass('::omero::grid::Repository', Repository, -1, (), True, False, None, (), ())
    Repository._ice_type = _M_omero.grid._t_Repository

    Repository._op_root = IcePy.Operation('root', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_OriginalFile, False, 0), (_M_omero._t_ServerError,))
    Repository._op_mimetype = IcePy.Operation('mimetype', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_string, False, 0), (_M_omero._t_ServerError,))
    Repository._op_list = IcePy.Operation('list', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), _M_omero.api._t_StringSet, False, 0), (_M_omero._t_ServerError,))
    Repository._op_listFiles = IcePy.Operation('listFiles', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), _M_omero.api._t_OriginalFileList, False, 0), (_M_omero._t_ServerError,))
    Repository._op_register = IcePy.Operation('register', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0), ((), _M_omero._t_RString, False, 0)), (), ((), _M_omero.model._t_OriginalFile, False, 0), (_M_omero._t_ServerError,))
    Repository._op_file = IcePy.Operation('file', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0), ((), IcePy._t_string, False, 0)), (), ((), _M_omero.api._t_RawFileStorePrx, False, 0), (_M_omero._t_ServerError,))
    Repository._op_fileById = IcePy.Operation('fileById', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.api._t_RawFileStorePrx, False, 0), (_M_omero._t_ServerError,))
    Repository._op_fileExists = IcePy.Operation('fileExists', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_ServerError,))
    Repository._op_makeDir = IcePy.Operation('makeDir', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0), ((), IcePy._t_bool, False, 0)), (), None, (_M_omero._t_ServerError,))
    Repository._op_treeList = IcePy.Operation('treeList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), _M_omero._t_RMap, False, 0), (_M_omero._t_ServerError,))
    Repository._op_deletePaths = IcePy.Operation('deletePaths', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.api._t_StringArray, False, 0), ((), IcePy._t_bool, False, 0), ((), IcePy._t_bool, False, 0)), (), ((), _M_omero.cmd._t_HandlePrx, False, 0), (_M_omero._t_ServerError,))

    _M_omero.grid.Repository = Repository
    del Repository

    _M_omero.grid.RepositoryPrx = RepositoryPrx
    del RepositoryPrx

if 'ImportLocation' not in _M_omero.grid.__dict__:
    _M_omero.grid.ImportLocation = Ice.createTempClass()
    class ImportLocation(Ice.Object):
        """
        Returned by {@code ManagedRepository.importFileset} with
        the information needed to proceed with an FS import.
        For the examples that follow, assume that the used
        files passed to importFileset were:
        /Users/jack/Documents/Data/Experiment-1/1.dv
        /Users/jack/Documents/Data/Experiment-1/1.dv.log
        /Users/jack/Documents/Data/Experiment-2/2.dv
        /Users/jack/Documents/Data/Experiment-2/2.dv.log
        Members:
        sharedPath -- The shared base of all the paths passed to
        the server.
        omittedLevels -- Number of directories which have been omitted
        from the original paths passed to the server.
        usedFiles -- Parsed string names which should be used by the
        clients during upload. This array will be of the
        same length as the argument passed to
        {@code ManagedRepository.importFileset} but will have
        shortened paths.
        Experiment/1.dv
        Experiment/1.dv.log
        directory -- Represents the directory to which all files
        will be uploaded.
        """
        def __init__(self, sharedPath='', omittedLevels=0, usedFiles=None, directory=None):
            self.sharedPath = sharedPath
            self.omittedLevels = omittedLevels
            self.usedFiles = usedFiles
            self.directory = directory

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::grid::ImportLocation')

        def ice_id(self, current=None):
            return '::omero::grid::ImportLocation'

        def ice_staticId():
            return '::omero::grid::ImportLocation'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid._t_ImportLocation)

        __repr__ = __str__

    _M_omero.grid.ImportLocationPrx = Ice.createTempClass()
    class ImportLocationPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.ImportLocationPrx.ice_checkedCast(proxy, '::omero::grid::ImportLocation', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.ImportLocationPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::ImportLocation'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid._t_ImportLocationPrx = IcePy.defineProxy('::omero::grid::ImportLocation', ImportLocationPrx)

    _M_omero.grid._t_ImportLocation = IcePy.declareClass('::omero::grid::ImportLocation')

    _M_omero.grid._t_ImportLocation = IcePy.defineClass('::omero::grid::ImportLocation', ImportLocation, -1, (), False, False, None, (), (
        ('sharedPath', (), IcePy._t_string, False, 0),
        ('omittedLevels', (), IcePy._t_int, False, 0),
        ('usedFiles', (), _M_omero.api._t_StringSet, False, 0),
        ('directory', (), _M_omero.model._t_OriginalFile, False, 0)
    ))
    ImportLocation._ice_type = _M_omero.grid._t_ImportLocation

    _M_omero.grid.ImportLocation = ImportLocation
    del ImportLocation

    _M_omero.grid.ImportLocationPrx = ImportLocationPrx
    del ImportLocationPrx

if 'ImportSettings' not in _M_omero.grid.__dict__:
    _M_omero.grid.ImportSettings = Ice.createTempClass()
    class ImportSettings(Ice.Object):
        """
        User configuration options. These are likely set in the UI
        before the import is initiated.
        Members:
        userSpecifiedTarget -- The container which this object should be added to.
        userSpecifiedName -- Custom name suggested by the user.
        userSpecifiedDescription -- Custom description suggested by the user.
        userSpecifiedPixels -- User choice of pixels sizes.
        userSpecifiedAnnotationList -- Annotations that the user
        doThumbnails -- Whether or not the thumbnailing action should be performed.
        noStatsInfo -- Whether we are to disable StatsInfo population.
        checksumAlgorithm -- User choice of checksum algorithm for verifying upload.
        """
        def __init__(self, userSpecifiedTarget=None, userSpecifiedName=None, userSpecifiedDescription=None, userSpecifiedPixels=None, userSpecifiedAnnotationList=None, doThumbnails=None, noStatsInfo=None, checksumAlgorithm=None):
            self.userSpecifiedTarget = userSpecifiedTarget
            self.userSpecifiedName = userSpecifiedName
            self.userSpecifiedDescription = userSpecifiedDescription
            self.userSpecifiedPixels = userSpecifiedPixels
            self.userSpecifiedAnnotationList = userSpecifiedAnnotationList
            self.doThumbnails = doThumbnails
            self.noStatsInfo = noStatsInfo
            self.checksumAlgorithm = checksumAlgorithm

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::grid::ImportSettings')

        def ice_id(self, current=None):
            return '::omero::grid::ImportSettings'

        def ice_staticId():
            return '::omero::grid::ImportSettings'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid._t_ImportSettings)

        __repr__ = __str__

    _M_omero.grid.ImportSettingsPrx = Ice.createTempClass()
    class ImportSettingsPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.ImportSettingsPrx.ice_checkedCast(proxy, '::omero::grid::ImportSettings', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.ImportSettingsPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::ImportSettings'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid._t_ImportSettingsPrx = IcePy.defineProxy('::omero::grid::ImportSettings', ImportSettingsPrx)

    _M_omero.grid._t_ImportSettings = IcePy.declareClass('::omero::grid::ImportSettings')

    _M_omero.grid._t_ImportSettings = IcePy.defineClass('::omero::grid::ImportSettings', ImportSettings, -1, (), False, False, None, (), (
        ('userSpecifiedTarget', (), _M_omero.model._t_IObject, False, 0),
        ('userSpecifiedName', (), _M_omero._t_RString, False, 0),
        ('userSpecifiedDescription', (), _M_omero._t_RString, False, 0),
        ('userSpecifiedPixels', (), _M_omero.api._t_DoubleArray, False, 0),
        ('userSpecifiedAnnotationList', (), _M_omero.api._t_AnnotationList, False, 0),
        ('doThumbnails', (), _M_omero._t_RBool, False, 0),
        ('noStatsInfo', (), _M_omero._t_RBool, False, 0),
        ('checksumAlgorithm', (), _M_omero.model._t_ChecksumAlgorithm, False, 0)
    ))
    ImportSettings._ice_type = _M_omero.grid._t_ImportSettings

    _M_omero.grid.ImportSettings = ImportSettings
    del ImportSettings

    _M_omero.grid.ImportSettingsPrx = ImportSettingsPrx
    del ImportSettingsPrx

if 'ImportProcess' not in _M_omero.grid.__dict__:
    _M_omero.grid.ImportProcess = Ice.createTempClass()
    class ImportProcess(_M_omero.api.StatefulServiceInterface):
        """
        User configuration options. These are likely set in the UI
        before the import is initiated.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.grid.ImportProcess:
                raise RuntimeError('omero.grid.ImportProcess is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::ServiceInterface', '::omero::api::StatefulServiceInterface', '::omero::grid::ImportProcess')

        def ice_id(self, current=None):
            return '::omero::grid::ImportProcess'

        def ice_staticId():
            return '::omero::grid::ImportProcess'
        ice_staticId = staticmethod(ice_staticId)

        def getUploader(self, i, current=None):
            """
            Step 1: Returns a RawFileStore that can be used to upload one of
            the used files. The index is the same as the used file listed in
            {@code ImportLocation}. {@code omero.api.RawFileStore.close}
            should be called once all data has been transferred. If the
            file must be re-written, call {@code getUploader} with the
            same index again. Once all uploads have been completed,
            {@code verifyUpload} should be called to initiate background
            processing
            Arguments:
            i -- 
            current -- The Current object for the invocation.
            """
            pass

        def verifyUpload(self, hash, current=None):
            """
            Step 2: Passes a set of client-side calculated hashes to the
            server for verifying that all of the files were correctly
            uploaded. If this passes then a omero.cmd.Handle
            proxy is returned, which completes all the necessary import
            steps. A successful import will return an
            {@code ImportResponse}. Otherwise, some omero.cmd.ERR
            will be returned.
            Arguments:
            hash -- 
            current -- The Current object for the invocation.
            """
            pass

        def getUploadOffset(self, i, current=None):
            """
            In case an upload must be resumed, this provides the
            location of the last successful upload.
            Arguments:
            i -- 
            current -- The Current object for the invocation.
            """
            pass

        def getHandle(self, current=None):
            """
            Reacquire the handle which was returned by
            {@code verifyUpload}. This is useful in case a new
            client is re-attaching to a running import.
            From the omero.cmd.Handle instance, the
            original {@code ImportRequest} can also be found.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def getImportSettings(self, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid._t_ImportProcess)

        __repr__ = __str__

    _M_omero.grid.ImportProcessPrx = Ice.createTempClass()
    class ImportProcessPrx(_M_omero.api.StatefulServiceInterfacePrx):

        """
        Step 1: Returns a RawFileStore that can be used to upload one of
        the used files. The index is the same as the used file listed in
        {@code ImportLocation}. {@code omero.api.RawFileStore.close}
        should be called once all data has been transferred. If the
        file must be re-written, call {@code getUploader} with the
        same index again. Once all uploads have been completed,
        {@code verifyUpload} should be called to initiate background
        processing
        Arguments:
        i -- 
        _ctx -- The request context for the invocation.
        """
        def getUploader(self, i, _ctx=None):
            return _M_omero.grid.ImportProcess._op_getUploader.invoke(self, ((i, ), _ctx))

        """
        Step 1: Returns a RawFileStore that can be used to upload one of
        the used files. The index is the same as the used file listed in
        {@code ImportLocation}. {@code omero.api.RawFileStore.close}
        should be called once all data has been transferred. If the
        file must be re-written, call {@code getUploader} with the
        same index again. Once all uploads have been completed,
        {@code verifyUpload} should be called to initiate background
        processing
        Arguments:
        i -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getUploader(self, i, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.ImportProcess._op_getUploader.begin(self, ((i, ), _response, _ex, _sent, _ctx))

        """
        Step 1: Returns a RawFileStore that can be used to upload one of
        the used files. The index is the same as the used file listed in
        {@code ImportLocation}. {@code omero.api.RawFileStore.close}
        should be called once all data has been transferred. If the
        file must be re-written, call {@code getUploader} with the
        same index again. Once all uploads have been completed,
        {@code verifyUpload} should be called to initiate background
        processing
        Arguments:
        i -- 
        """
        def end_getUploader(self, _r):
            return _M_omero.grid.ImportProcess._op_getUploader.end(self, _r)

        """
        Step 2: Passes a set of client-side calculated hashes to the
        server for verifying that all of the files were correctly
        uploaded. If this passes then a omero.cmd.Handle
        proxy is returned, which completes all the necessary import
        steps. A successful import will return an
        {@code ImportResponse}. Otherwise, some omero.cmd.ERR
        will be returned.
        Arguments:
        hash -- 
        _ctx -- The request context for the invocation.
        """
        def verifyUpload(self, hash, _ctx=None):
            return _M_omero.grid.ImportProcess._op_verifyUpload.invoke(self, ((hash, ), _ctx))

        """
        Step 2: Passes a set of client-side calculated hashes to the
        server for verifying that all of the files were correctly
        uploaded. If this passes then a omero.cmd.Handle
        proxy is returned, which completes all the necessary import
        steps. A successful import will return an
        {@code ImportResponse}. Otherwise, some omero.cmd.ERR
        will be returned.
        Arguments:
        hash -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_verifyUpload(self, hash, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.ImportProcess._op_verifyUpload.begin(self, ((hash, ), _response, _ex, _sent, _ctx))

        """
        Step 2: Passes a set of client-side calculated hashes to the
        server for verifying that all of the files were correctly
        uploaded. If this passes then a omero.cmd.Handle
        proxy is returned, which completes all the necessary import
        steps. A successful import will return an
        {@code ImportResponse}. Otherwise, some omero.cmd.ERR
        will be returned.
        Arguments:
        hash -- 
        """
        def end_verifyUpload(self, _r):
            return _M_omero.grid.ImportProcess._op_verifyUpload.end(self, _r)

        """
        In case an upload must be resumed, this provides the
        location of the last successful upload.
        Arguments:
        i -- 
        _ctx -- The request context for the invocation.
        """
        def getUploadOffset(self, i, _ctx=None):
            return _M_omero.grid.ImportProcess._op_getUploadOffset.invoke(self, ((i, ), _ctx))

        """
        In case an upload must be resumed, this provides the
        location of the last successful upload.
        Arguments:
        i -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getUploadOffset(self, i, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.ImportProcess._op_getUploadOffset.begin(self, ((i, ), _response, _ex, _sent, _ctx))

        """
        In case an upload must be resumed, this provides the
        location of the last successful upload.
        Arguments:
        i -- 
        """
        def end_getUploadOffset(self, _r):
            return _M_omero.grid.ImportProcess._op_getUploadOffset.end(self, _r)

        """
        Reacquire the handle which was returned by
        {@code verifyUpload}. This is useful in case a new
        client is re-attaching to a running import.
        From the omero.cmd.Handle instance, the
        original {@code ImportRequest} can also be found.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def getHandle(self, _ctx=None):
            return _M_omero.grid.ImportProcess._op_getHandle.invoke(self, ((), _ctx))

        """
        Reacquire the handle which was returned by
        {@code verifyUpload}. This is useful in case a new
        client is re-attaching to a running import.
        From the omero.cmd.Handle instance, the
        original {@code ImportRequest} can also be found.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getHandle(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.ImportProcess._op_getHandle.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Reacquire the handle which was returned by
        {@code verifyUpload}. This is useful in case a new
        client is re-attaching to a running import.
        From the omero.cmd.Handle instance, the
        original {@code ImportRequest} can also be found.
        Arguments:
        """
        def end_getHandle(self, _r):
            return _M_omero.grid.ImportProcess._op_getHandle.end(self, _r)

        def getImportSettings(self, _ctx=None):
            return _M_omero.grid.ImportProcess._op_getImportSettings.invoke(self, ((), _ctx))

        def begin_getImportSettings(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.ImportProcess._op_getImportSettings.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getImportSettings(self, _r):
            return _M_omero.grid.ImportProcess._op_getImportSettings.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.ImportProcessPrx.ice_checkedCast(proxy, '::omero::grid::ImportProcess', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.ImportProcessPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::ImportProcess'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid._t_ImportProcessPrx = IcePy.defineProxy('::omero::grid::ImportProcess', ImportProcessPrx)

    _M_omero.grid._t_ImportProcess = IcePy.defineClass('::omero::grid::ImportProcess', ImportProcess, -1, (), True, False, None, (_M_omero.api._t_StatefulServiceInterface,), ())
    ImportProcess._ice_type = _M_omero.grid._t_ImportProcess

    ImportProcess._op_getUploader = IcePy.Operation('getUploader', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_int, False, 0),), (), ((), _M_omero.api._t_RawFileStorePrx, False, 0), (_M_omero._t_ServerError,))
    ImportProcess._op_verifyUpload = IcePy.Operation('verifyUpload', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.api._t_StringSet, False, 0),), (), ((), _M_omero.cmd._t_HandlePrx, False, 0), (_M_omero._t_ServerError,))
    ImportProcess._op_getUploadOffset = IcePy.Operation('getUploadOffset', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_int, False, 0),), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    ImportProcess._op_getHandle = IcePy.Operation('getHandle', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.cmd._t_HandlePrx, False, 0), (_M_omero._t_ServerError,))
    ImportProcess._op_getImportSettings = IcePy.Operation('getImportSettings', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.grid._t_ImportSettings, False, 0), ())

    _M_omero.grid.ImportProcess = ImportProcess
    del ImportProcess

    _M_omero.grid.ImportProcessPrx = ImportProcessPrx
    del ImportProcessPrx

if '_t_ImportProcessList' not in _M_omero.grid.__dict__:
    _M_omero.grid._t_ImportProcessList = IcePy.defineSequence('::omero::grid::ImportProcessList', (), _M_omero.grid._t_ImportProcessPrx)

if 'ImportRequest' not in _M_omero.grid.__dict__:
    _M_omero.grid.ImportRequest = Ice.createTempClass()
    class ImportRequest(_M_omero.cmd.Request):
        """
        Command object which will be used to create
        the omero.cmd.Handle instances passed
        back by the {@code ImportProcess}.
        Members:
        clientUuid -- Lookup value for the session that import is taking
        part in.
        repoUuid -- Repository which is responsible for this import.
        All files which are uploaded will be available
        from it.
        process -- Proxy of the process which this request
        will be running in. This value will be
        filled in for possible later re-use, but
        is not read by the server.
        activity -- Activity that this will be filling
        out in the database. This always points to a
        omero.model.MetadataImportJob which is the
        first server-side phase after the
        omero.model.UploadJob.
        settings -- ImportSettings which are provided by the
        client on the call to {@code ManagedRepository.importFileset}.
        location -- ImportLocation which is calculated during
        the call to {@code ManagedRepository.importFileset}.
        logFile -- omero.model.OriginalFile object representing the import log file.
        """
        def __init__(self, clientUuid='', repoUuid='', process=None, activity=None, settings=None, location=None, logFile=None):
            _M_omero.cmd.Request.__init__(self)
            self.clientUuid = clientUuid
            self.repoUuid = repoUuid
            self.process = process
            self.activity = activity
            self.settings = settings
            self.location = location
            self.logFile = logFile

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::Request', '::omero::grid::ImportRequest')

        def ice_id(self, current=None):
            return '::omero::grid::ImportRequest'

        def ice_staticId():
            return '::omero::grid::ImportRequest'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid._t_ImportRequest)

        __repr__ = __str__

    _M_omero.grid.ImportRequestPrx = Ice.createTempClass()
    class ImportRequestPrx(_M_omero.cmd.RequestPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.ImportRequestPrx.ice_checkedCast(proxy, '::omero::grid::ImportRequest', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.ImportRequestPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::ImportRequest'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid._t_ImportRequestPrx = IcePy.defineProxy('::omero::grid::ImportRequest', ImportRequestPrx)

    _M_omero.grid._t_ImportRequest = IcePy.declareClass('::omero::grid::ImportRequest')

    _M_omero.grid._t_ImportRequest = IcePy.defineClass('::omero::grid::ImportRequest', ImportRequest, -1, (), False, False, _M_omero.cmd._t_Request, (), (
        ('clientUuid', (), IcePy._t_string, False, 0),
        ('repoUuid', (), IcePy._t_string, False, 0),
        ('process', (), _M_omero.grid._t_ImportProcessPrx, False, 0),
        ('activity', (), _M_omero.model._t_FilesetJobLink, False, 0),
        ('settings', (), _M_omero.grid._t_ImportSettings, False, 0),
        ('location', (), _M_omero.grid._t_ImportLocation, False, 0),
        ('logFile', (), _M_omero.model._t_OriginalFile, False, 0)
    ))
    ImportRequest._ice_type = _M_omero.grid._t_ImportRequest

    _M_omero.grid.ImportRequest = ImportRequest
    del ImportRequest

    _M_omero.grid.ImportRequestPrx = ImportRequestPrx
    del ImportRequestPrx

if 'ImportResponse' not in _M_omero.grid.__dict__:
    _M_omero.grid.ImportResponse = Ice.createTempClass()
    class ImportResponse(_M_omero.cmd.OK):
        """
        Successful response returned from execution
        of ImportRequest. This is the simplest way
        to return the results, but is likely not the
        overall best strategy.
        Members:
        pixels -- 
        objects -- Top-level OME-XML objects which are created
        during the import. This will not contain any
        pixels which were imported, but images, plates,
        etc. which may be useful for user feedback.
        """
        def __init__(self, pixels=None, objects=None):
            _M_omero.cmd.OK.__init__(self)
            self.pixels = pixels
            self.objects = objects

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::OK', '::omero::cmd::Response', '::omero::grid::ImportResponse')

        def ice_id(self, current=None):
            return '::omero::grid::ImportResponse'

        def ice_staticId():
            return '::omero::grid::ImportResponse'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid._t_ImportResponse)

        __repr__ = __str__

    _M_omero.grid.ImportResponsePrx = Ice.createTempClass()
    class ImportResponsePrx(_M_omero.cmd.OKPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.ImportResponsePrx.ice_checkedCast(proxy, '::omero::grid::ImportResponse', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.ImportResponsePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::ImportResponse'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid._t_ImportResponsePrx = IcePy.defineProxy('::omero::grid::ImportResponse', ImportResponsePrx)

    _M_omero.grid._t_ImportResponse = IcePy.declareClass('::omero::grid::ImportResponse')

    _M_omero.grid._t_ImportResponse = IcePy.defineClass('::omero::grid::ImportResponse', ImportResponse, -1, (), False, False, _M_omero.cmd._t_OK, (), (
        ('pixels', (), _M_omero.api._t_PixelsList, False, 0),
        ('objects', (), _M_omero.api._t_IObjectList, False, 0)
    ))
    ImportResponse._ice_type = _M_omero.grid._t_ImportResponse

    _M_omero.grid.ImportResponse = ImportResponse
    del ImportResponse

    _M_omero.grid.ImportResponsePrx = ImportResponsePrx
    del ImportResponsePrx

if 'ManagedRepository' not in _M_omero.grid.__dict__:
    _M_omero.grid.ManagedRepository = Ice.createTempClass()
    class ManagedRepository(_M_omero.grid.Repository):
        """
        FS-enabled repository which can convert uploaded files
        into Images by using Bio-Formats to import them.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.grid.ManagedRepository:
                raise RuntimeError('omero.grid.ManagedRepository is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::grid::ManagedRepository', '::omero::grid::Repository')

        def ice_id(self, current=None):
            return '::omero::grid::ManagedRepository'

        def ice_staticId():
            return '::omero::grid::ManagedRepository'
        ice_staticId = staticmethod(ice_staticId)

        def importFileset(self, fs, settings, current=None):
            """
            Returns an ImportProcess which can be used to upload files.
            On ImportProcess#verifyUpload, an omero.cmd.Handle will be
            returned which can be watched for knowing when the server-side import
            is complete.
            Client paths set in the fileset entries must /-separate their components.
            Once the upload is complete, the ImportProcess must be closed.
            Once omero.cmd.Handle#getResponse returns a non-null value, the
            handle instance can and must be closed.
            Arguments:
            fs -- 
            settings -- 
            current -- The Current object for the invocation.
            """
            pass

        def importPaths(self, filePaths, current=None):
            """
            For clients without access to Bio-Formats, the simplified
            {@code importPaths} method allows passing solely the absolute
            path of the files to be uploaded (no directories) and all
            configuration happens server-side. Much of the functionality
            provided via omero.model.Fileset and
            omero.grid.ImportSettings is of course lost.
            Arguments:
            filePaths -- 
            current -- The Current object for the invocation.
            """
            pass

        def listImports(self, current=None):
            """
            List imports that are currently running in this importer.
            These will be limited based on user/group membership for
            the omero.model.Fileset object which is being created
            by the import. If the user has write permissions for the
            fileset, then the import will be included.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def listChecksumAlgorithms(self, current=None):
            """
            Return the list of checksum algorithms supported by this repository
            for verifying the integrity of uploaded files.
            They are named as ""algorithm-integer"",
            integer being the bit width of the resulting hash code.
            It is possible for the same algorithm to be offered with
            different bit widths.
            They are listed in descending order of preference,
            as set by the server administrator, and any of them may
            be specified for ImportSettings#checksumAlgorithm.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def suggestChecksumAlgorithm(self, supported, current=None):
            """
            Suggest a checksum algorithm to use for
            ImportSettings#checksumAlgorithm according to the
            preferences set by the server administrator. Provide a
            list of the algorithms supported by the client, and the
            server will report which of them is most preferred by
            the server, or return null if none of them are supported.
            Arguments:
            supported -- 
            current -- The Current object for the invocation.
            """
            pass

        def verifyChecksums(self, ids, current=None):
            """
            Verify the checksum for the original files identified by
            the given IDs.
            The files must be in this repository.
            Returns the IDs of the original files whose checksums
            do not match the file on disk.
            Arguments:
            ids -- 
            current -- The Current object for the invocation.
            """
            pass

        def setChecksumAlgorithm(self, hasher, ids, current=None):
            """
            Set the checksum algorithm for the original files identified
            by the given IDs and calculate their checksum accordingly.
            The files must be in this repository.
            Existing checksums are checked before being changed.
            If a checksum does not match, ServerError will be thrown;
            in this case some other files may already have had their
            checksum algorithm set.
            Returns the IDs of the original files that did not already
            have a checksum set for the given algorithm.
            Arguments:
            hasher -- 
            ids -- 
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid._t_ManagedRepository)

        __repr__ = __str__

    _M_omero.grid.ManagedRepositoryPrx = Ice.createTempClass()
    class ManagedRepositoryPrx(_M_omero.grid.RepositoryPrx):

        """
        Returns an ImportProcess which can be used to upload files.
        On ImportProcess#verifyUpload, an omero.cmd.Handle will be
        returned which can be watched for knowing when the server-side import
        is complete.
        Client paths set in the fileset entries must /-separate their components.
        Once the upload is complete, the ImportProcess must be closed.
        Once omero.cmd.Handle#getResponse returns a non-null value, the
        handle instance can and must be closed.
        Arguments:
        fs -- 
        settings -- 
        _ctx -- The request context for the invocation.
        """
        def importFileset(self, fs, settings, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_importFileset.invoke(self, ((fs, settings), _ctx))

        """
        Returns an ImportProcess which can be used to upload files.
        On ImportProcess#verifyUpload, an omero.cmd.Handle will be
        returned which can be watched for knowing when the server-side import
        is complete.
        Client paths set in the fileset entries must /-separate their components.
        Once the upload is complete, the ImportProcess must be closed.
        Once omero.cmd.Handle#getResponse returns a non-null value, the
        handle instance can and must be closed.
        Arguments:
        fs -- 
        settings -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_importFileset(self, fs, settings, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_importFileset.begin(self, ((fs, settings), _response, _ex, _sent, _ctx))

        """
        Returns an ImportProcess which can be used to upload files.
        On ImportProcess#verifyUpload, an omero.cmd.Handle will be
        returned which can be watched for knowing when the server-side import
        is complete.
        Client paths set in the fileset entries must /-separate their components.
        Once the upload is complete, the ImportProcess must be closed.
        Once omero.cmd.Handle#getResponse returns a non-null value, the
        handle instance can and must be closed.
        Arguments:
        fs -- 
        settings -- 
        """
        def end_importFileset(self, _r):
            return _M_omero.grid.ManagedRepository._op_importFileset.end(self, _r)

        """
        For clients without access to Bio-Formats, the simplified
        {@code importPaths} method allows passing solely the absolute
        path of the files to be uploaded (no directories) and all
        configuration happens server-side. Much of the functionality
        provided via omero.model.Fileset and
        omero.grid.ImportSettings is of course lost.
        Arguments:
        filePaths -- 
        _ctx -- The request context for the invocation.
        """
        def importPaths(self, filePaths, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_importPaths.invoke(self, ((filePaths, ), _ctx))

        """
        For clients without access to Bio-Formats, the simplified
        {@code importPaths} method allows passing solely the absolute
        path of the files to be uploaded (no directories) and all
        configuration happens server-side. Much of the functionality
        provided via omero.model.Fileset and
        omero.grid.ImportSettings is of course lost.
        Arguments:
        filePaths -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_importPaths(self, filePaths, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_importPaths.begin(self, ((filePaths, ), _response, _ex, _sent, _ctx))

        """
        For clients without access to Bio-Formats, the simplified
        {@code importPaths} method allows passing solely the absolute
        path of the files to be uploaded (no directories) and all
        configuration happens server-side. Much of the functionality
        provided via omero.model.Fileset and
        omero.grid.ImportSettings is of course lost.
        Arguments:
        filePaths -- 
        """
        def end_importPaths(self, _r):
            return _M_omero.grid.ManagedRepository._op_importPaths.end(self, _r)

        """
        List imports that are currently running in this importer.
        These will be limited based on user/group membership for
        the omero.model.Fileset object which is being created
        by the import. If the user has write permissions for the
        fileset, then the import will be included.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def listImports(self, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_listImports.invoke(self, ((), _ctx))

        """
        List imports that are currently running in this importer.
        These will be limited based on user/group membership for
        the omero.model.Fileset object which is being created
        by the import. If the user has write permissions for the
        fileset, then the import will be included.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_listImports(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_listImports.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        List imports that are currently running in this importer.
        These will be limited based on user/group membership for
        the omero.model.Fileset object which is being created
        by the import. If the user has write permissions for the
        fileset, then the import will be included.
        Arguments:
        """
        def end_listImports(self, _r):
            return _M_omero.grid.ManagedRepository._op_listImports.end(self, _r)

        """
        Return the list of checksum algorithms supported by this repository
        for verifying the integrity of uploaded files.
        They are named as ""algorithm-integer"",
        integer being the bit width of the resulting hash code.
        It is possible for the same algorithm to be offered with
        different bit widths.
        They are listed in descending order of preference,
        as set by the server administrator, and any of them may
        be specified for ImportSettings#checksumAlgorithm.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def listChecksumAlgorithms(self, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_listChecksumAlgorithms.invoke(self, ((), _ctx))

        """
        Return the list of checksum algorithms supported by this repository
        for verifying the integrity of uploaded files.
        They are named as ""algorithm-integer"",
        integer being the bit width of the resulting hash code.
        It is possible for the same algorithm to be offered with
        different bit widths.
        They are listed in descending order of preference,
        as set by the server administrator, and any of them may
        be specified for ImportSettings#checksumAlgorithm.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_listChecksumAlgorithms(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_listChecksumAlgorithms.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Return the list of checksum algorithms supported by this repository
        for verifying the integrity of uploaded files.
        They are named as ""algorithm-integer"",
        integer being the bit width of the resulting hash code.
        It is possible for the same algorithm to be offered with
        different bit widths.
        They are listed in descending order of preference,
        as set by the server administrator, and any of them may
        be specified for ImportSettings#checksumAlgorithm.
        Arguments:
        """
        def end_listChecksumAlgorithms(self, _r):
            return _M_omero.grid.ManagedRepository._op_listChecksumAlgorithms.end(self, _r)

        """
        Suggest a checksum algorithm to use for
        ImportSettings#checksumAlgorithm according to the
        preferences set by the server administrator. Provide a
        list of the algorithms supported by the client, and the
        server will report which of them is most preferred by
        the server, or return null if none of them are supported.
        Arguments:
        supported -- 
        _ctx -- The request context for the invocation.
        """
        def suggestChecksumAlgorithm(self, supported, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_suggestChecksumAlgorithm.invoke(self, ((supported, ), _ctx))

        """
        Suggest a checksum algorithm to use for
        ImportSettings#checksumAlgorithm according to the
        preferences set by the server administrator. Provide a
        list of the algorithms supported by the client, and the
        server will report which of them is most preferred by
        the server, or return null if none of them are supported.
        Arguments:
        supported -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_suggestChecksumAlgorithm(self, supported, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_suggestChecksumAlgorithm.begin(self, ((supported, ), _response, _ex, _sent, _ctx))

        """
        Suggest a checksum algorithm to use for
        ImportSettings#checksumAlgorithm according to the
        preferences set by the server administrator. Provide a
        list of the algorithms supported by the client, and the
        server will report which of them is most preferred by
        the server, or return null if none of them are supported.
        Arguments:
        supported -- 
        """
        def end_suggestChecksumAlgorithm(self, _r):
            return _M_omero.grid.ManagedRepository._op_suggestChecksumAlgorithm.end(self, _r)

        """
        Verify the checksum for the original files identified by
        the given IDs.
        The files must be in this repository.
        Returns the IDs of the original files whose checksums
        do not match the file on disk.
        Arguments:
        ids -- 
        _ctx -- The request context for the invocation.
        """
        def verifyChecksums(self, ids, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_verifyChecksums.invoke(self, ((ids, ), _ctx))

        """
        Verify the checksum for the original files identified by
        the given IDs.
        The files must be in this repository.
        Returns the IDs of the original files whose checksums
        do not match the file on disk.
        Arguments:
        ids -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_verifyChecksums(self, ids, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_verifyChecksums.begin(self, ((ids, ), _response, _ex, _sent, _ctx))

        """
        Verify the checksum for the original files identified by
        the given IDs.
        The files must be in this repository.
        Returns the IDs of the original files whose checksums
        do not match the file on disk.
        Arguments:
        ids -- 
        """
        def end_verifyChecksums(self, _r):
            return _M_omero.grid.ManagedRepository._op_verifyChecksums.end(self, _r)

        """
        Set the checksum algorithm for the original files identified
        by the given IDs and calculate their checksum accordingly.
        The files must be in this repository.
        Existing checksums are checked before being changed.
        If a checksum does not match, ServerError will be thrown;
        in this case some other files may already have had their
        checksum algorithm set.
        Returns the IDs of the original files that did not already
        have a checksum set for the given algorithm.
        Arguments:
        hasher -- 
        ids -- 
        _ctx -- The request context for the invocation.
        """
        def setChecksumAlgorithm(self, hasher, ids, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_setChecksumAlgorithm.invoke(self, ((hasher, ids), _ctx))

        """
        Set the checksum algorithm for the original files identified
        by the given IDs and calculate their checksum accordingly.
        The files must be in this repository.
        Existing checksums are checked before being changed.
        If a checksum does not match, ServerError will be thrown;
        in this case some other files may already have had their
        checksum algorithm set.
        Returns the IDs of the original files that did not already
        have a checksum set for the given algorithm.
        Arguments:
        hasher -- 
        ids -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setChecksumAlgorithm(self, hasher, ids, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.ManagedRepository._op_setChecksumAlgorithm.begin(self, ((hasher, ids), _response, _ex, _sent, _ctx))

        """
        Set the checksum algorithm for the original files identified
        by the given IDs and calculate their checksum accordingly.
        The files must be in this repository.
        Existing checksums are checked before being changed.
        If a checksum does not match, ServerError will be thrown;
        in this case some other files may already have had their
        checksum algorithm set.
        Returns the IDs of the original files that did not already
        have a checksum set for the given algorithm.
        Arguments:
        hasher -- 
        ids -- 
        """
        def end_setChecksumAlgorithm(self, _r):
            return _M_omero.grid.ManagedRepository._op_setChecksumAlgorithm.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.ManagedRepositoryPrx.ice_checkedCast(proxy, '::omero::grid::ManagedRepository', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.ManagedRepositoryPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::ManagedRepository'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid._t_ManagedRepositoryPrx = IcePy.defineProxy('::omero::grid::ManagedRepository', ManagedRepositoryPrx)

    _M_omero.grid._t_ManagedRepository = IcePy.defineClass('::omero::grid::ManagedRepository', ManagedRepository, -1, (), True, False, None, (_M_omero.grid._t_Repository,), ())
    ManagedRepository._ice_type = _M_omero.grid._t_ManagedRepository

    ManagedRepository._op_importFileset = IcePy.Operation('importFileset', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Fileset, False, 0), ((), _M_omero.grid._t_ImportSettings, False, 0)), (), ((), _M_omero.grid._t_ImportProcessPrx, False, 0), (_M_omero._t_ServerError,))
    ManagedRepository._op_importPaths = IcePy.Operation('importPaths', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.api._t_StringSet, False, 0),), (), ((), _M_omero.grid._t_ImportProcessPrx, False, 0), (_M_omero._t_ServerError,))
    ManagedRepository._op_listImports = IcePy.Operation('listImports', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.grid._t_ImportProcessList, False, 0), (_M_omero._t_ServerError,))
    ManagedRepository._op_listChecksumAlgorithms = IcePy.Operation('listChecksumAlgorithms', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_ChecksumAlgorithmList, False, 0), ())
    ManagedRepository._op_suggestChecksumAlgorithm = IcePy.Operation('suggestChecksumAlgorithm', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.api._t_ChecksumAlgorithmList, False, 0),), (), ((), _M_omero.model._t_ChecksumAlgorithm, False, 0), ())
    ManagedRepository._op_verifyChecksums = IcePy.Operation('verifyChecksums', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.api._t_LongList, False, 0),), (), ((), _M_omero.api._t_LongList, False, 0), (_M_omero._t_ServerError,))
    ManagedRepository._op_setChecksumAlgorithm = IcePy.Operation('setChecksumAlgorithm', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ChecksumAlgorithm, False, 0), ((), _M_omero.api._t_LongList, False, 0)), (), ((), _M_omero.api._t_LongList, False, 0), (_M_omero._t_ServerError,))

    _M_omero.grid.ManagedRepository = ManagedRepository
    del ManagedRepository

    _M_omero.grid.ManagedRepositoryPrx = ManagedRepositoryPrx
    del ManagedRepositoryPrx

if 'RawAccessRequest' not in _M_omero.grid.__dict__:
    _M_omero.grid.RawAccessRequest = Ice.createTempClass()
    class RawAccessRequest(_M_omero.cmd.Request):
        """
        Command object which will be parsed by the internal
        repository given by ""repo"". This command will *only*
        be processed if the user has sufficient rights (e.g.
        is a member of "system") and is largely intended for
        testing and diagnosis rather than actual client
        functionality.
        """
        def __init__(self, repoUuid='', command='', args=None, path=''):
            _M_omero.cmd.Request.__init__(self)
            self.repoUuid = repoUuid
            self.command = command
            self.args = args
            self.path = path

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::Request', '::omero::grid::RawAccessRequest')

        def ice_id(self, current=None):
            return '::omero::grid::RawAccessRequest'

        def ice_staticId():
            return '::omero::grid::RawAccessRequest'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid._t_RawAccessRequest)

        __repr__ = __str__

    _M_omero.grid.RawAccessRequestPrx = Ice.createTempClass()
    class RawAccessRequestPrx(_M_omero.cmd.RequestPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.RawAccessRequestPrx.ice_checkedCast(proxy, '::omero::grid::RawAccessRequest', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.RawAccessRequestPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::RawAccessRequest'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid._t_RawAccessRequestPrx = IcePy.defineProxy('::omero::grid::RawAccessRequest', RawAccessRequestPrx)

    _M_omero.grid._t_RawAccessRequest = IcePy.defineClass('::omero::grid::RawAccessRequest', RawAccessRequest, -1, (), False, False, _M_omero.cmd._t_Request, (), (
        ('repoUuid', (), IcePy._t_string, False, 0),
        ('command', (), IcePy._t_string, False, 0),
        ('args', (), _M_omero.api._t_StringSet, False, 0),
        ('path', (), IcePy._t_string, False, 0)
    ))
    RawAccessRequest._ice_type = _M_omero.grid._t_RawAccessRequest

    _M_omero.grid.RawAccessRequest = RawAccessRequest
    del RawAccessRequest

    _M_omero.grid.RawAccessRequestPrx = RawAccessRequestPrx
    del RawAccessRequestPrx

if 'InternalRepository' not in _M_omero.grid.__dict__:
    _M_omero.grid.InternalRepository = Ice.createTempClass()
    class InternalRepository(Ice.Object):
        """
        Internal portion of the API used for management. Not available to clients.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.grid.InternalRepository:
                raise RuntimeError('omero.grid.InternalRepository is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::grid::InternalRepository')

        def ice_id(self, current=None):
            return '::omero::grid::InternalRepository'

        def ice_staticId():
            return '::omero::grid::InternalRepository'
        ice_staticId = staticmethod(ice_staticId)

        def createRawFileStore(self, file, current=None):
            pass

        def createRawPixelsStore(self, file, current=None):
            pass

        def createRenderingEngine(self, file, current=None):
            pass

        def createThumbnailStore(self, file, current=None):
            pass

        def getDescription(self, current=None):
            pass

        def getProxy(self, current=None):
            pass

        def rawAccess(self, raw, current=None):
            pass

        def getFilePath(self, file, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid._t_InternalRepository)

        __repr__ = __str__

    _M_omero.grid.InternalRepositoryPrx = Ice.createTempClass()
    class InternalRepositoryPrx(Ice.ObjectPrx):

        def createRawFileStore(self, file, _ctx=None):
            return _M_omero.grid.InternalRepository._op_createRawFileStore.invoke(self, ((file, ), _ctx))

        def begin_createRawFileStore(self, file, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.InternalRepository._op_createRawFileStore.begin(self, ((file, ), _response, _ex, _sent, _ctx))

        def end_createRawFileStore(self, _r):
            return _M_omero.grid.InternalRepository._op_createRawFileStore.end(self, _r)

        def createRawPixelsStore(self, file, _ctx=None):
            return _M_omero.grid.InternalRepository._op_createRawPixelsStore.invoke(self, ((file, ), _ctx))

        def begin_createRawPixelsStore(self, file, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.InternalRepository._op_createRawPixelsStore.begin(self, ((file, ), _response, _ex, _sent, _ctx))

        def end_createRawPixelsStore(self, _r):
            return _M_omero.grid.InternalRepository._op_createRawPixelsStore.end(self, _r)

        def createRenderingEngine(self, file, _ctx=None):
            return _M_omero.grid.InternalRepository._op_createRenderingEngine.invoke(self, ((file, ), _ctx))

        def begin_createRenderingEngine(self, file, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.InternalRepository._op_createRenderingEngine.begin(self, ((file, ), _response, _ex, _sent, _ctx))

        def end_createRenderingEngine(self, _r):
            return _M_omero.grid.InternalRepository._op_createRenderingEngine.end(self, _r)

        def createThumbnailStore(self, file, _ctx=None):
            return _M_omero.grid.InternalRepository._op_createThumbnailStore.invoke(self, ((file, ), _ctx))

        def begin_createThumbnailStore(self, file, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.InternalRepository._op_createThumbnailStore.begin(self, ((file, ), _response, _ex, _sent, _ctx))

        def end_createThumbnailStore(self, _r):
            return _M_omero.grid.InternalRepository._op_createThumbnailStore.end(self, _r)

        def getDescription(self, _ctx=None):
            return _M_omero.grid.InternalRepository._op_getDescription.invoke(self, ((), _ctx))

        def begin_getDescription(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.InternalRepository._op_getDescription.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getDescription(self, _r):
            return _M_omero.grid.InternalRepository._op_getDescription.end(self, _r)

        def getProxy(self, _ctx=None):
            return _M_omero.grid.InternalRepository._op_getProxy.invoke(self, ((), _ctx))

        def begin_getProxy(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.InternalRepository._op_getProxy.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getProxy(self, _r):
            return _M_omero.grid.InternalRepository._op_getProxy.end(self, _r)

        def rawAccess(self, raw, _ctx=None):
            return _M_omero.grid.InternalRepository._op_rawAccess.invoke(self, ((raw, ), _ctx))

        def begin_rawAccess(self, raw, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.InternalRepository._op_rawAccess.begin(self, ((raw, ), _response, _ex, _sent, _ctx))

        def end_rawAccess(self, _r):
            return _M_omero.grid.InternalRepository._op_rawAccess.end(self, _r)

        def getFilePath(self, file, _ctx=None):
            return _M_omero.grid.InternalRepository._op_getFilePath.invoke(self, ((file, ), _ctx))

        def begin_getFilePath(self, file, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.InternalRepository._op_getFilePath.begin(self, ((file, ), _response, _ex, _sent, _ctx))

        def end_getFilePath(self, _r):
            return _M_omero.grid.InternalRepository._op_getFilePath.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.InternalRepositoryPrx.ice_checkedCast(proxy, '::omero::grid::InternalRepository', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.InternalRepositoryPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::InternalRepository'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid._t_InternalRepositoryPrx = IcePy.defineProxy('::omero::grid::InternalRepository', InternalRepositoryPrx)

    _M_omero.grid._t_InternalRepository = IcePy.defineClass('::omero::grid::InternalRepository', InternalRepository, -1, (), True, False, None, (), ())
    InternalRepository._ice_type = _M_omero.grid._t_InternalRepository

    InternalRepository._op_createRawFileStore = IcePy.Operation('createRawFileStore', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_OriginalFile, False, 0),), (), ((), _M_omero.api._t_RawFileStorePrx, False, 0), (_M_omero._t_ServerError,))
    InternalRepository._op_createRawPixelsStore = IcePy.Operation('createRawPixelsStore', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_OriginalFile, False, 0),), (), ((), _M_omero.api._t_RawPixelsStorePrx, False, 0), (_M_omero._t_ServerError,))
    InternalRepository._op_createRenderingEngine = IcePy.Operation('createRenderingEngine', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_OriginalFile, False, 0),), (), ((), _M_omero.api._t_RenderingEnginePrx, False, 0), (_M_omero._t_ServerError,))
    InternalRepository._op_createThumbnailStore = IcePy.Operation('createThumbnailStore', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_OriginalFile, False, 0),), (), ((), _M_omero.api._t_ThumbnailStorePrx, False, 0), (_M_omero._t_ServerError,))
    InternalRepository._op_getDescription = IcePy.Operation('getDescription', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_OriginalFile, False, 0), (_M_omero._t_ServerError,))
    InternalRepository._op_getProxy = IcePy.Operation('getProxy', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.grid._t_RepositoryPrx, False, 0), (_M_omero._t_ServerError,))
    InternalRepository._op_rawAccess = IcePy.Operation('rawAccess', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.grid._t_RawAccessRequest, False, 0),), (), ((), _M_omero.cmd._t_Response, False, 0), (_M_omero._t_ServerError,))
    InternalRepository._op_getFilePath = IcePy.Operation('getFilePath', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_OriginalFile, False, 0),), (), ((), IcePy._t_string, False, 0), (_M_omero._t_ServerError,))

    _M_omero.grid.InternalRepository = InternalRepository
    del InternalRepository

    _M_omero.grid.InternalRepositoryPrx = InternalRepositoryPrx
    del InternalRepositoryPrx

if '_t_RepositoryProxyList' not in _M_omero.grid.__dict__:
    _M_omero.grid._t_RepositoryProxyList = IcePy.defineSequence('::omero::grid::RepositoryProxyList', (), _M_omero.grid._t_RepositoryPrx)

if 'RepositoryMap' not in _M_omero.grid.__dict__:
    _M_omero.grid.RepositoryMap = Ice.createTempClass()
    class RepositoryMap(object):
        """
        Return value for omero.grid.SharedResources#repositories
        The descriptions and proxies arrays will have the same size and each
        index in descriptions (non-null) will match a possibly null proxy, if
        the given repository is not currently accessible.
        """
        def __init__(self, descriptions=None, proxies=None):
            self.descriptions = descriptions
            self.proxies = proxies

        def __eq__(self, other):
            if other is None:
                return False
            elif not isinstance(other, _M_omero.grid.RepositoryMap):
                return NotImplemented
            else:
                if self.descriptions != other.descriptions:
                    return False
                if self.proxies != other.proxies:
                    return False
                return True

        def __ne__(self, other):
            return not self.__eq__(other)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid._t_RepositoryMap)

        __repr__ = __str__

    _M_omero.grid._t_RepositoryMap = IcePy.defineStruct('::omero::grid::RepositoryMap', RepositoryMap, (), (
        ('descriptions', (), _M_omero.api._t_OriginalFileList),
        ('proxies', (), _M_omero.grid._t_RepositoryProxyList)
    ))

    _M_omero.grid.RepositoryMap = RepositoryMap
    del RepositoryMap

# End of module omero.grid

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# http://svn.smedbergs.us/python-processes/trunk/killableprocess.py
#

# killableprocess - subprocesses which can be reliably killed
#
# Parts of this module are copied from the subprocess.py file contained
# in the Python distribution.
#
# Copyright (c) 2003-2004 by Peter Astrand <astrand@lysator.liu.se>
#
# Additions and modifications written by Benjamin Smedberg
# <benjamin@smedbergs.us> are Copyright (c) 2006 by the Mozilla Foundation
# <http://www.mozilla.org/>
#
# By obtaining, using, and/or copying this software and/or its
# associated documentation, you agree that you have read, understood,
# and will comply with the following terms and conditions:
#
# Permission to use, copy, modify, and distribute this software and
# its associated documentation for any purpose and without fee is
# hereby granted, provided that the above copyright notice appears in
# all copies, and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of the
# author not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
# WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

r"""killableprocess - Subprocesses which can be reliably killed

This module is a subclass of the builtin "subprocess" module. It allows
processes that launch subprocesses to be reliably killed on Windows (via the Popen.kill() method.

It also adds a timeout argument to Wait() for a limited period of time before
forcefully killing the process.

Note: On Windows, this module requires Windows 2000 or higher (no support for
Windows 95, 98, or NT 4.0).
"""
from __future__ import absolute_import

import subprocess
import sys
import os
import time
import types
from subprocess import CalledProcessError

mswindows = (sys.platform == "win32")

if mswindows:
    from . import winprocess
else:
    import signal

def call(*args, **kwargs):
    waitargs = {}
    if "timeout" in kwargs:
        waitargs["timeout"] = kwargs.pop("timeout")

    return Popen(*args, **kwargs).wait(**waitargs)

def check_call(*args, **kwargs):
    """Call a program with an optional timeout. If the program has a non-zero
    exit status, raises a CalledProcessError."""

    retcode = call(*args, **kwargs)
    if retcode:
        cmd = kwargs.get("args")
        if cmd is None:
            cmd = args[0]
        raise CalledProcessError(retcode, cmd)

if not mswindows:
    def DoNothing(*args):
        pass

class Popen(subprocess.Popen):
    if not mswindows:
        # Override __init__ to set a preexec_fn
        def __init__(self, *args, **kwargs):
            if len(args) >= 7:
                raise Exception("Arguments preexec_fn and after must be passed by keyword.")

            real_preexec_fn = kwargs.pop("preexec_fn", None)
            def setpgid_preexec_fn():
                os.setpgid(0, 0)
                if real_preexec_fn:
                    apply(real_preexec_fn)

            kwargs['preexec_fn'] = setpgid_preexec_fn

            subprocess.Popen.__init__(self, *args, **kwargs)

    if mswindows:
        def _execute_child(self, *args_tuple):
            # The arguments to this internal Python function changed on
            # Windows in 2.7.6
            # - https://trac.openmicroscopy.org/ome/ticket/12320
            # Upstream bug and fix:
            # - https://bugzilla.mozilla.org/show_bug.cgi?id=958609
            # - https://github.com/mozilla/addon-sdk/pull/1379
            if sys.hexversion < 0x02070600: # prior to 2.7.6
                (args, executable, preexec_fn, close_fds,
                    cwd, env, universal_newlines, startupinfo,
                    creationflags, shell,
                    p2cread, p2cwrite,
                    c2pread, c2pwrite,
                    errread, errwrite) = args_tuple
                to_close = set()
            else: # 2.7.6 and later
                (args, executable, preexec_fn, close_fds,
                    cwd, env, universal_newlines, startupinfo,
                    creationflags, shell, to_close,
                    p2cread, p2cwrite,
                    c2pread, c2pwrite,
                    errread, errwrite) = args_tuple

            if not isinstance(args, (str,)):
                args = subprocess.list2cmdline(args)

            if startupinfo is None:
                startupinfo = winprocess.STARTUPINFO()

            if None not in (p2cread, c2pwrite, errwrite):
                startupinfo.dwFlags |= winprocess.STARTF_USESTDHANDLES
                
                startupinfo.hStdInput = int(p2cread)
                startupinfo.hStdOutput = int(c2pwrite)
                startupinfo.hStdError = int(errwrite)
            if shell:
                startupinfo.dwFlags |= winprocess.STARTF_USESHOWWINDOW
                startupinfo.wShowWindow = winprocess.SW_HIDE
                comspec = os.environ.get("COMSPEC", "cmd.exe")
                args = comspec + " /c " + args

            # We create a new job for this process, so that we can kill
            # the process and any sub-processes 
            self._job = winprocess.CreateJobObject()

            creationflags |= winprocess.CREATE_SUSPENDED
            creationflags |= winprocess.CREATE_UNICODE_ENVIRONMENT

            hp, ht, pid, tid = winprocess.CreateProcess(
                executable, args,
                None, None, # No special security
                1, # Must inherit handles!
                creationflags,
                winprocess.EnvironmentBlock(env),
                cwd, startupinfo)
            
            self._child_created = True
            self._handle = hp
            self._thread = ht
            self.pid = pid

            winprocess.AssignProcessToJobObject(self._job, hp)
            winprocess.ResumeThread(ht)

            if p2cread is not None:
                p2cread.Close()
            if c2pwrite is not None:
                c2pwrite.Close()
            if errwrite is not None:
                errwrite.Close()

    def kill(self, group=True):
        """Kill the process. If group=True, all sub-processes will also be killed."""
        if mswindows:
            if group:
                winprocess.TerminateJobObject(self._job, 127)
            else:
                winprocess.TerminateProcess(self._handle, 127)
            self.returncode = 127    
        else:
            if group:
                os.killpg(self.pid, signal.SIGKILL)
            else:
                os.kill(self.pid, signal.SIGKILL)
            self.returncode = -9

    def wait(self, timeout=-1, group=True):
        """Wait for the process to terminate. Returns returncode attribute.
        If timeout seconds are reached and the process has not terminated,
        it will be forcefully killed. If timeout is -1, wait will not
        time out."""

        if self.returncode is not None:
            return self.returncode

        if mswindows:
            if timeout != -1:
                timeout = timeout * 1000
            rc = winprocess.WaitForSingleObject(self._handle, timeout)
            if rc == winprocess.WAIT_TIMEOUT:
                self.kill(group)
            else:
                self.returncode = winprocess.GetExitCodeProcess(self._handle)
        else:
            if timeout == -1:
                subprocess.Popen.wait(self)
                return self.returncode
            
            starttime = time.time()

            # Make sure there is a signal handler for SIGCHLD installed
            oldsignal = signal.signal(signal.SIGCHLD, DoNothing)

            while time.time() < starttime + timeout - 0.01:
                pid, sts = os.waitpid(self.pid, os.WNOHANG)
                if pid != 0:
                    self._handle_exitstatus(sts)
                    signal.signal(signal.SIGCHLD, oldsignal)
                    return self.returncode
                
                # time.sleep is interrupted by signals (good!)
                newtimeout = timeout - time.time() + starttime
                time.sleep(newtimeout)

            self.kill(group)
            signal.signal(signal.SIGCHLD, oldsignal)
            subprocess.Popen.wait(self)

        return self.returncode



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2008 Glencoe Software, Inc.  All Rights Reserved.
# Use is subject to license terms supplied in LICENSE.txt
#

"""
OMERO Grid Processor
"""

from __future__ import division
from builtins import str
from builtins import range
from future.utils import native_str
from past.utils import old_div
from builtins import object
import os
import time
import signal
import uuid
from omero_ext import killableprocess as subprocess


import Ice
import omero
import omero.clients
import omero.scripts
import omero.util
import omero.util.concurrency

from omero.util import load_dotted_class
from omero.util.temp_files import create_path, remove_path
from omero.util.decorators import remoted, perf, locked, wraps
from omero.rtypes import rint, rlong
from omero_ext.path import path

sys = __import__("sys")


def with_context(func, context):
    """ Decorator for invoking Ice methods with a context """
    def handler(*args, **kwargs):
        args = list(args)
        args.append(context)
        return func(*args, **kwargs)
    handler = wraps(func)(handler)
    return handler


class WithGroup(object):
    """
    Wraps a ServiceInterfacePrx instance and applies
    a "omero.group" to the passed context on every
    invocation.

    For example, using a job handle as root requires logging
    manually into the group. (ticket:2044)
    """

    def __init__(self, service, group_id):
        self._service = service
        self._group_id = native_str(group_id)

    def _get_ctx(self, group=None):
        ctx = self._service.ice_getCommunicator()\
            .getImplicitContext().getContext()
        ctx = dict(ctx)
        ctx["omero.group"] = native_str(group)
        return ctx

    def __getattr__(self, name):
        if name.startswith("_"):
            return self.__dict__[name]
        elif hasattr(self._service, name):
            method = getattr(self._service, name)
            ctx = self._get_ctx(self._group_id)
            return with_context(method, ctx)
        raise AttributeError(
            "'%s' object has no attribute '%s'" % (self.service, name))


class ProcessI(omero.grid.Process, omero.util.SimpleServant):
    """
    Wrapper around a subprocess.Popen instance. Returned by ProcessorI
    when a job is submitted. This implementation uses the given
    interpreter to call a file that must be named "script" in the
    generated temporary directory.

    Call is equivalent to:

    cd TMP_DIR
    ICE_CONFIG=./config interpreter ./script >out 2>err &

    The properties argument is used to generate the ./config file.

    The params argument may be null in which case this process
    is being used solely to calculate the parameters for the script
    ("omero.scripts.parse=true")

    If iskill is True, then on cleanup, this process will reap the
    attached session completely.
    """

    def __init__(self, ctx, interpreter, properties, params, iskill=False,
                 Popen=subprocess.Popen,
                 callback_cast=omero.grid.ProcessCallbackPrx.uncheckedCast,
                 omero_home=path.getcwd()):
        """
        Popen and callback_Cast are primarily for testing.
        """
        omero.util.SimpleServant.__init__(self, ctx)
        self.omero_home = omero_home  #: Location for OMERO_HOME/lib/python
        #: Executable which will be used on the script
        self.interpreter = interpreter
        #: Properties used to create an Ice.Config
        self.properties = properties
        #: JobParams for this script. Possibly None if a ParseJob
        self.params = params
        #: Whether or not, cleanup should kill the session
        self.iskill = iskill
        #: Function which should be used for creating processes
        self.Popen = Popen
        #: Function used to cast all ProcessCallback proxies
        self.callback_cast = callback_cast
        # Non arguments (mutable state)
        self.rcode = None  #: return code from popen
        self.callbacks = {}  #: dictionary from id strings to callback proxies
        self.popen = None  #: process. if None then this instance isn't alive.
        self.pid = None  #: pid of the process. Once set, isn't nulled.
        self.started = None  #: time the process started
        self.stopped = None  #: time of deactivation
        #: status which will be sent on set_job_status
        self.final_status = None
        # Non arguments (immutable state)
        #: session this instance is tied to
        self.uuid = properties["omero.user"]

        # More fields set by these methods
        self.make_files()
        self.make_env()
        self.make_config()
        self.logger.info("Created %s in %s" % (self.uuid, self.dir))

    #
    # Initialization methods
    #

    def make_env(self):
        self.env = omero.util.Environment(
            "CLASSPATH",
            "DISPLAY",
            "DYLD_LIBRARY_PATH",
            "HOME",
            "JYTHON_HOME",
            "LC_ALL",
            "LANG",
            "LANGUAGE",
            "LD_LIBRARY_PATH",
            "MLABRAW_CMD_STR",
            "OMERODIR",
            "OMERO_TEMPDIR",
            "OMERO_TMPDIR",
            "PATH",
            "PYTHONPATH",
        )

        # Since we know the location of our OMERO, we're going to
        # force the value for OMERO_HOME. This is useful in scripts
        # which want to be able to find their location.
        self.env.set("OMERO_HOME", self.omero_home)

        # WORKAROUND
        # Currently duplicating the logic here as in the PYTHONPATH
        # setting of the grid application descriptor (see etc/grid/*.xml)
        # This should actually be taken care of in the descriptor itself
        # by having setting PYTHONPATH to an absolute value. This is
        # not currently possible with IceGrid (without using icepatch --
        # see 39.17.2 "node.datadir).
        self.env.append("PYTHONPATH", str(self.omero_home / "lib" / "python"))
        self.env.set("ICE_CONFIG", str(self.config_path))
        # Also actively adding all jars under lib/server to the CLASSPATH
        lib_server = self.omero_home / "lib" / "server"
        for jar_file in lib_server.walk("*.jar"):
            self.env.append("CLASSPATH", str(jar_file))

    def make_files(self):
        self.dir = create_path("process", ".dir", folder=True)
        self.script_path = old_div(self.dir, "script")
        self.config_path = old_div(self.dir, "config")
        self.stdout_path = old_div(self.dir, "out")
        self.stderr_path = old_div(self.dir, "err")

    def make_config(self):
        """
        Creates the ICE_CONFIG file used by the client.
        """
        config_file = open(str(self.config_path), "w")
        try:
            for key in self.properties.keys():
                config_file.write("%s=%s\n" % (key, self.properties[key]))
        finally:
            config_file.close()

    def tmp_client(self):
        """
        Create a client for performing cleanup operations.
        This client should be closed as soon as possible
        by the process
        """
        try:
            client = omero.client(["--Ice.Config=%s" % str(self.config_path)])
            client.setAgent("OMERO.process")
            client.createSession().detachOnDestroy()
            self.logger.debug("client: %s" % client.sf)
            return client
        except:
            self.logger.error("Failed to create client for %s" % self.uuid)
            return None

    #
    # Activation / Deactivation
    #

    @locked
    def activate(self):
        """
        Process creation has to wait until all external downloads, etc
        are finished.
        """

        if self.isActive():
            raise omero.ApiUsageException(None, None, "Already activated")

        self.stdout = open(str(self.stdout_path), "w")
        self.stderr = open(str(self.stderr_path), "w")
        self.popen = self.Popen(
            self.command(),
            cwd=str(self.dir), env=self.env(),
            stdout=self.stdout, stderr=self.stderr)
        self.pid = self.popen.pid
        self.started = time.time()
        self.stopped = None
        self.status("Activated")

    def command(self):
        """
        Method to allow subclasses to override the launch
        behavior by changing the command passed to self.Popen
        """
        return [self.interpreter, "./script"]

    @locked
    def deactivate(self):
        """
        Cleans up the temporary directory used by the process, and terminates
        the Popen process if running.
        """

        if not self.isActive():
            raise omero.ApiUsageException(None, None, "Not active")

        if self.stopped:
            # Prevent recursion since we are reusing kill & cancel
            return

        self.stopped = time.time()
        d_start = time.time()
        self.status("Deactivating")

        # None of these should throw, but just in case
        try:

            self.shutdown()     # Calls cancel & kill which recall this method!
            self.popen = None   # Now we are finished

            client = self.tmp_client()
            try:
                self.set_job_status(client)
                self.cleanup_output()
                self.upload_output(client)  # Important!
                self.cleanup_tmpdir()
            finally:
                if client:
                    client.__del__()  # Safe closeSession

        except Exception:
            self.logger.error(
                "FAILED TO CLEANUP pid=%s (%s)",
                self.pid, self.uuid, exc_info=True)

        d_stop = time.time()
        elapsed = int(self.stopped - self.started)
        d_elapsed = int(d_stop - d_start)
        self.status("Lived %ss. Deactivation took %ss." % (elapsed, d_elapsed))

    @locked
    def isActive(self):
        """
        Tests only if this instance has a non-None popen attribute. After
        activation this method will return True until the popen itself returns
        a non-None value (self.rcode) at which time it will be nulled and this
        method will again return False
        """
        return self.popen is not None

    @locked
    def wasActivated(self):
        """
        Returns true only if this instance has either a non-null
        popen or a non-null rcode field.
        """
        return self.popen is not None or self.rcode is not None

    @locked
    def isRunning(self):
        return self.popen is not None and self.rcode is None

    @locked
    def isFinished(self):
        return self.rcode is not None

    @locked
    def alreadyDone(self):
        """
        Allows short-cutting various checks if we already
        have a rcode for this popen. A non-None return value
        implies that a process was started and returned
        the given non-None value itself.
        """
        if not self.wasActivated:
            raise omero.InternalException(
                None, None, "Process never activated")
        return self.isFinished()

    #
    # Cleanup methods
    #

    def __del__(self):
        self.cleanup()

    @perf
    @locked
    def check(self):
        """
        Called periodically to keep the session alive. Returns
        False if this resource can be cleaned up. (Resources API)
        """

        if not self.wasActivated():
            return True  # This should only happen on startup, so ignore

        try:
            self.poll()
            self.ctx.getSession().getSessionService().getSession(self.uuid)
            return True
        except:
            self.status("Keep alive failed")
            return False

    @perf
    @locked
    def cleanup(self):
        """
        Deactivates the process (if active) and cleanups the server
        connection. (Resources API)
        """

        if self.isRunning():
            self.deactivate()

        if not self.iskill:
            return

        try:
            sf = self.ctx.getSession(recreate=False)
        except:
            self.logger.debug("Can't get session for cleanup")
            return

        self.status("Killing session")
        svc = sf.getSessionService()
        obj = omero.model.SessionI()
        obj.uuid = omero.rtypes.rstring(self.uuid)
        try:
            while svc.closeSession(obj) > 0:
                pass
            # No action to be taken when iskill == False if
            # we don't have an actual client to worry with.
        except:
            self.logger.error(
                "Error on session cleanup, kill=%s" %
                self.iskill, exc_info=True)

    def cleanup_output(self):
        """
        Flush and close the stderr and stdout streams.
        """
        try:
            if hasattr(self, "stderr"):
                self.stderr.flush()
                self.stderr.close()
        except:
            self.logger.error("cleanup of sterr failed", exc_info=True)
        try:
            if hasattr(self, "stdout"):
                self.stdout.flush()
                self.stdout.close()
        except:
            self.logger.error("cleanup of sterr failed", exc_info=True)

    def set_job_status(self, client):
        """
        Sets the job status
        """
        if not client:
            self.logger.error(
                "No client: Cannot set job status for pid=%s (%s)",
                self.pid, self.uuid)
            return

        gid = client.sf.getAdminService().getEventContext().groupId
        handle = WithGroup(client.sf.createJobHandle(), gid)
        try:
            status = self.final_status
            if status is None:
                status = (self.rcode == 0 and "Finished" or "Error")
            handle.attach(int(self.properties["omero.job"]))
            oldStatus = handle.setStatus(status)
            self.status(
                "Changed job status from %s to %s" % (oldStatus, status))
        finally:
            handle.close()

    def upload_output(self, client):
        """
        If this is not a params calculation (i.e. parms != null) and the
        stdout or stderr are non-null, they they will be uploaded and
        attached to the job.
        """
        if not client:
            self.logger.error(
                "No client: Cannot upload output for pid=%s (%s)",
                self.pid, self.uuid)
            return

        if self.params:
            out_format = self.params.stdoutFormat
            err_format = self.params.stderrFormat
        else:
            out_format = "text/plain"
            err_format = out_format

        self._upload(client, self.stdout_path, "stdout", out_format)
        self._upload(client, self.stderr_path, "stderr", err_format)

    def _upload(self, client, filename, name, format):

        if not format:
            return

        filename = str(filename)  # Might be path.path
        sz = os.path.getsize(filename)
        if not sz:
            self.status("No %s" % name)
            return

        try:
            ofile = client.upload(filename, name=name, type=format)
            jobid = int(client.getProperty("omero.job"))
            link = omero.model.JobOriginalFileLinkI()
            if self.params is None:
                link.parent = omero.model.ParseJobI(rlong(jobid), False)
            else:
                link.parent = omero.model.ScriptJobI(rlong(jobid), False)
            link.child = ofile.proxy()
            client.getSession().getUpdateService().saveObject(link)
            self.status(
                "Uploaded %s bytes of %s to %s" %
                (sz, filename, ofile.id.val))
        except:
            self.logger.error(
                "Error on upload of %s for pid=%s (%s)",
                filename, self.pid, self.uuid, exc_info=True)

    def cleanup_tmpdir(self):
        """
        Remove all known files and finally the temporary directory.
        If other files exist, an exception will be raised.
        """
        try:
            remove_path(self.dir)
        except:
            self.logger.error(
                "Failed to remove dir %s" % self.dir, exc_info=True)

    #
    # popen methods
    #

    def status(self, msg=""):
        if self.isRunning():
            self.rcode = self.popen.poll()
        self.logger.info("%s : %s", self, msg)

    @perf
    @remoted
    def poll(self, current=None):
        """
        Checks popen.poll() (if active) and notifies all callbacks
        if necessary. If this method returns a non-None value, then
        the process will be marked inactive.
        """

        if self.alreadyDone():
            return rint(self.rcode)

        self.status("Polling")
        if self.rcode is None:
            # Haven't finished yet, so do nothing.
            return None
        else:
            self.deactivate()
            rv = rint(self.rcode)
            self.allcallbacks("processFinished", self.rcode)
            return rv

    @perf
    @remoted
    def wait(self, current=None):
        """
        Waits on popen.wait() to return (if active) and notifies
        all callbacks. Marks this process as inactive.
        """

        if self.alreadyDone():
            return self.rcode

        self.status("Waiting")
        self.rcode = self.popen.wait()
        self.deactivate()
        self.allcallbacks("processFinished", self.rcode)
        return self.rcode

    def _term(self):
        """
        Attempts to cancel the process by sending SIGTERM
        (or similar)
        """
        try:
            self.status("os.kill(TERM)")
            os.kill(self.popen.pid, signal.SIGTERM)
        except AttributeError:
            self.logger.debug("No os.kill(TERM). Skipping cancel")

    def _send(self, iskill):
        """
        Helper method for sending signals. This method only
        makes a call is the process is active.
        """
        if self.isRunning():
            try:
                if self.popen.poll() is None:
                    if iskill:
                        self.status("popen.kill(True)")
                        self.popen.kill(True)
                    else:
                        self._term()

                else:
                    self.status("Skipped signal")
            except OSError as oserr:
                self.logger.debug(
                    "err on pid=%s iskill=%s : %s", self.popen.pid, iskill,
                    oserr)

    @perf
    @remoted
    def cancel(self, current=None):
        """
        Tries to cancel popen (if active) and notifies callbacks.
        """

        if self.alreadyDone():
            return True

        self.final_status = "Cancelled"
        self._send(iskill=False)
        finished = self.isFinished()
        if finished:
            self.deactivate()
        self.allcallbacks("processCancelled", finished)
        return finished

    @perf
    @remoted
    def kill(self, current=None):

        if self.alreadyDone():
            return True

        self.final_status = "Cancelled"
        self._send(iskill=True)
        finished = self.isFinished()
        if finished:
            self.deactivate()
        self.allcallbacks("processKilled", finished)
        return finished

    @perf
    @remoted
    def shutdown(self, current=None):
        """
        If self.popen is active, then first call cancel, wait a period of
        time, and finally call kill.
        """

        if self.alreadyDone():
            return

        self.status("Shutdown")
        try:
            for i in range(5, 0, -1):
                if self.cancel():
                    break
                else:
                    self.logger.warning(
                        "Shutdown: %s (%s). Killing in %s seconds.", self.pid,
                        self.uuid, 6*(i-1)+1)
                    self.stop_event.wait(6)
            self.kill()
        except:
            self.logger.error(
                "Shutdown failed: %s (%s)", self.pid, self.uuid,
                exc_info=True)

    #
    # Callbacks
    #

    @remoted
    @locked
    def registerCallback(self, callback, current=None):
        try:
            id = callback.ice_getIdentity()
            key = "%s/%s" % (id.category, id.name)
            callback = callback.ice_oneway()
            callback = self.callback_cast(callback)
            if not callback:
                e = "Callback is invalid"
            else:
                self.callbacks[key] = callback
                self.logger.debug("Added callback: %s", key)
                return
        except Exception as ex:
            e = ex
        # Only reached on failure
        msg = "Failed to add callback: %s. Reason: %s" % (callback, e)
        self.logger.debug(msg)
        raise omero.ApiUsageException(None, None, msg)

    @remoted
    @locked
    def unregisterCallback(self, callback, current=None):
        try:
            id = callback.ice_getIdentity()
            key = "%s/%s" % (id.category, id.name)
            if key not in self.callback:
                raise omero.ApiUsageException(
                    None, None, "No callback registered with id: %s" % key)
            del self.callbacks[key]
            self.logger.debug("Removed callback: %s", key)
        except Exception as e:
            msg = "Failed to remove callback: %s. Reason: %s" % (callback, e)
            self.logger.debug(msg)
            raise omero.ApiUsageException(None, None, msg)

    @locked
    def allcallbacks(self, method, arg):
        self.status("Callback %s" % method)
        for key, cb in list(self.callbacks.items()):
            try:
                m = getattr(cb, method)
                m(arg)
            except Ice.LocalException:
                self.logger.debug(
                    "LocalException calling callback %s on pid=%s (%s)"
                    % (key, self.pid, self.uuid), exc_info=False)
            except:
                self.logger.error(
                    "Error calling callback %s on pid=%s (%s)"
                    % (key, self.pid, self.uuid), exc_info=True)

    def __str__(self):
        return "<proc:%s,rc=%s,uuid=%s>" % (
            self.pid, (self.rcode is None and "-" or self.rcode), self.uuid)


class MATLABProcessI(ProcessI):

    def make_files(self):
        """
        Modify the script_path field from ProcessI.make_files
        in ordert to append a ".m"
        """
        ProcessI.make_files(self)
        self.script_path = old_div(self.dir, "script.m")

    def command(self):
        """
        Overrides ProcessI to call MATLAB idiosyncratically.
        """
        r = "try, cd('%s'); script; " % self.dir
        r += "catch e, disp(e.identifier); disp(e.message); exit(1); "
        r += "end, exit(0)"
        matlab_cmd = [
            self.interpreter, "-nosplash", "-nodisplay", "-nodesktop", "-r", r,
        ]
        return matlab_cmd


class UseSessionHolder(object):

    def __init__(self, sf):
        self.sf = sf

    def check(self):
        try:
            self.sf.keepAlive(None)
            return True
        except:
            return False

    def cleanup(self):
        pass


class ProcessorI(omero.grid.Processor, omero.util.Servant):

    def __init__(self, ctx, needs_session=True, use_session=None,
                 accepts_list=None, cfg=None, omero_home=path.getcwd(),
                 category=None):

        if accepts_list is None:
            accepts_list = []

        self.category = category  #: Category to be used w/ ProcessI
        self.omero_home = omero_home

        # Extensions for user-mode processors (ticket:1672)

        self.use_session = use_session
        """
        If set, this session will be returned from internal_session and
        the "needs_session" setting ignored.
        """

        if self.use_session:
            needs_session = False

        self.accepts_list = accepts_list
        """
        A list of contexts which will be accepted by this user-mode
        processor.
        """

        omero.util.Servant.__init__(self, ctx, needs_session=needs_session)
        if cfg is None:
            self.cfg = os.path.join(omero_home, "etc", "ice.config")
            self.cfg = os.path.abspath(self.cfg)
        else:
            self.cfg = cfg

        # Keep this session alive until the processor is finished
        self.resources.add(UseSessionHolder(use_session))

    def setProxy(self, prx):
        """
        Overrides the default action in order to register this proxy
        with the session's sharedResources to register for callbacks.
        The on_newsession handler will also keep new sessions informed.

        See ticket:2304
        """
        omero.util.Servant.setProxy(self, prx)
        session = self.internal_session()
        self.register_session(session)

        # Keep other session informed
        self.ctx.on_newsession = self.register_session

    def user_client(self, agent):
        """
        Creates an omero.client instance for use by
        users.
        """
        args = ["--Ice.Config=%s" % (self.cfg)]
        rtr = self.internal_session().ice_getRouter()
        if rtr:
            # FIXME : How do we find an internal router?
            args.insert(0, "--Ice.Default.Router=%s" % rtr)
        client = omero.client(args)
        client.setAgent(agent)
        return client

    def internal_session(self):
        """
        Returns the session which should be used for lookups by this instance.
        Some methods will create a session based on the session parameter.
        In these cases, the session will belong to the user who is running a
        script.
        """
        if self.use_session:
            return self.use_session
        else:
            return self.ctx.getSession()

    def register_session(self, session):
        self.logger.info("Registering processor %s", self.prx)
        prx = omero.grid.ProcessorPrx.uncheckedCast(self.prx)
        session.sharedResources().addProcessor(prx)

    def lookup(self, job):
        sf = self.internal_session()
        gid = job.details.group.id.val
        handle = WithGroup(sf.createJobHandle(), gid)
        try:
            handle.attach(job.id.val)
            if handle.jobFinished():
                handle.close()
                raise omero.ApiUsageException("Job already finished.")

            prx = WithGroup(sf.getScriptService(), gid)
            file = prx.validateScript(job, self.accepts_list)

        except omero.SecurityViolation:
            self.logger.debug(
                "SecurityViolation on validate job %s from group %s",
                job.id.val, gid)
            file = None

        return file, handle

    @remoted
    def willAccept(self, userContext, groupContext, scriptContext, cb,
                   current=None):

        userID = None
        if userContext is not None:
            userID = userContext.id.val

        groupID = None
        if groupContext is not None:
            groupID = groupContext.id.val

        scriptID = None
        if scriptContext is not None:
            scriptID = scriptContext.id.val

        if scriptID:
            try:
                file, handle = self.lookup(scriptContext)
                handle.close()
                valid = (file is not None)
            except:
                self.logger.error(
                    "File lookup failed: user=%s, group=%s, script=%s",
                    userID, groupID, scriptID, exc_info=1)
                return  # EARlY EXIT !
        else:
            valid = False
            for x in self.accepts_list:
                if isinstance(x, omero.model.Experimenter) and \
                        x.id.val == userID:
                    valid = True
                elif isinstance(x, omero.model.ExperimenterGroup) and \
                        x.id.val == groupID:
                    valid = True

        self.logger.debug(
            "Accepts called on: user:%s group:%s scriptjob:%s - Valid: %s",
            userID, groupID, scriptID, valid)

        try:
            id = self.internal_session().ice_getIdentity().name
            cb = cb.ice_oneway()
            cb = omero.grid.ProcessorCallbackPrx.uncheckedCast(cb)
            prx = omero.grid.ProcessorPrx.uncheckedCast(self.prx)
            cb.isProxyAccepted(valid, id, prx)
        except Exception as e:
            self.logger.warn(
                "callback failed on willAccept: %s Exception:%s", cb, e)

        return valid

    @remoted
    def requestRunning(self, cb, current=None):

        try:
            cb = cb.ice_oneway()
            cb = omero.grid.ProcessorCallbackPrx.uncheckedCast(cb)
            servants = list(self.ctx.servant_map.values())
            rv = []

            for x in servants:
                try:
                    rv.append(int(x.properties["omero.job"]))
                except:
                    pass
            cb.responseRunning(rv)
        except Exception as e:
            self.logger.warn(
                "callback failed on requestRunning: %s Exception:%s", cb, e)

    @remoted
    def parseJob(self, session, job, current=None):
        self.logger.info(
            "parseJob: Session = %s, JobId = %s" % (session, job.id.val))
        client = self.user_client("OMERO.parseJob")

        try:
            iskill = False
            client.joinSession(session).detachOnDestroy()
            properties = {}
            properties["omero.scripts.parse"] = "true"
            prx, process = self.process(
                client, session, job, current, None, properties, iskill)
            process.wait()
            rv = client.getOutput("omero.scripts.parse")
            if rv is not None:
                return rv.val
            else:
                self.logger.warning(
                    "No output found for omero.scripts.parse. Keys: %s"
                    % client.getOutputKeys())
                return None
        finally:
            client.closeSession()
            del client

    @remoted
    def processJob(self, session, params, job, current=None):
        """
        """
        self.logger.info("processJob: Session = %s, JobId = %s"
                         % (session, job.id.val))
        client = self.user_client("OMERO.processJob")
        try:
            client.joinSession(session).detachOnDestroy()
            prx, process = self.process(
                client, session, job, current, params, iskill=True)
            return prx
        finally:
            client.closeSession()
            del client

    @perf
    def process(self, client, session, job, current, params, properties=None,
                iskill=True):
        """
        session: session uuid, used primarily if client is None
        client: an omero.client object which should be attached to a session
        """

        if properties is None:
            properties = {}

        if not session or not job or not job.id:
            raise omero.ApiUsageException("No null arguments")

        file, handle = self.lookup(job)

        try:
            if not file:
                raise omero.ApiUsageException(
                    None, None,
                    "Job should have one executable file attached.")

            sf = self.internal_session()
            if params:
                self.logger.debug("Checking params for job %s" % job.id.val)
                svc = sf.getSessionService()
                inputs = svc.getInputs(session)
                errors = omero.scripts.validate_inputs(
                    params, inputs, svc, session)
                if errors:
                    errors = "Invalid parameters:\n%s" % errors
                    raise omero.ValidationException(None, None, errors)

            properties["omero.job"] = str(job.id.val)
            properties["omero.user"] = session
            properties["omero.pass"] = session
            properties["Ice.Default.Router"] = \
                client.getProperty("Ice.Default.Router")

            launcher, ProcessClass = self.find_launcher(current)
            process = ProcessClass(self.ctx, launcher, properties, params,
                                   iskill, omero_home=self.omero_home)
            self.resources.add(process)

            # client.download(file, str(process.script_path))
            scriptText = sf.getScriptService().getScriptText(file.id.val)
            process.script_path.write_bytes(scriptText.encode('utf-8'))

            self.logger.info("Downloaded file: %s" % file.id.val)
            s = client.sha1(str(process.script_path))
            if not s == file.hash.val:
                msg = "Sha1s don't match! expected %s, found %s" \
                    % (file.hash.val, s)
                self.logger.error(msg)
                process.cleanup()
                raise omero.InternalException(None, None, msg)
            else:
                process.activate()
                handle.setStatus("Running")

            id = None
            if self.category:
                id = Ice.Identity()
                id.name = "Process-%s" % uuid.uuid4()
                id.category = self.category
            prx = self.ctx.add_servant(current, process, ice_identity=id)
            return omero.grid.ProcessPrx.uncheckedCast(prx), process

        finally:
            handle.close()

    def find_launcher(self, current):
        launcher = ""
        process_class = ""
        if current.ctx:
            launcher = current.ctx.get("omero.launcher", "")
            process_class = current.ctx.get(
                "omero.process", "omero.process.ProcessI")

        if not launcher:
            launcher = sys.executable

        self.logger.info("Using launcher: %s", launcher)
        self.logger.info("Using process: %s", process_class)

        # Imports in omero.util don't work well for this class
        # Handling classes from this module specially.
        internal = False
        parts = process_class.split(".")
        if len(parts) == 3:
            if parts[0:2] == ("omero", "processor"):
                internal = True

        if not process_class:
            ProcessClass = ProcessI
        elif internal:
            ProcessClass = globals()[parts[-1]]
        else:
            ProcessClass = load_dotted_class(process_class)
        return launcher, ProcessClass


def usermode_processor(client, serverid="UsermodeProcessor", cfg=None,
                       accepts_list=None, stop_event=None,
                       omero_home=path.getcwd()):
    """
    Creates and activates a usermode processor for the given client.
    It is the responsibility of the client to call "cleanup()" on
    the ProcessorI implementation which is returned.

    cfg is the path to an --Ice.Config-valid file or files. If none
    is given, the value of ICE_CONFIG will be taken from the environment
    if available. Otherwise, all properties will be taken from the client
    instance.

    accepts_list is the list of IObject instances which will be passed to
    omero.api.IScripts.validateScript. If none is given, only the current
    Experimenter's own object will be passed.

    stop_event is an threading.Event. One will be acquired from
    omero.util.concurrency.get_event if none is provided.
    """

    if cfg is None:
        cfg = os.environ.get("ICE_CONFIG")

    if accepts_list is None:
        uid = client.sf.getAdminService().getEventContext().userId
        accepts_list = [omero.model.ExperimenterI(uid, False)]

    if stop_event is None:
        stop_event = omero.util.concurrency.get_event(name="UsermodeProcessor")

    id = Ice.Identity()
    id.name = "%s-%s" % (serverid, uuid.uuid4())
    id.category = client.getCategory()

    ctx = omero.util.ServerContext(serverid, client.ic, stop_event)
    impl = omero.processor.ProcessorI(
        ctx, use_session=client.sf, accepts_list=accepts_list, cfg=cfg,
        omero_home=omero_home, category=id.category)
    ctx.add_servant(client.adapter, impl, ice_identity=id)
    return impl
